@@ -1,9 +1,23 @@
+# ********************************************************************
+#
+#  $Id: helloworld.py 32630 2018-10-10 14:11:07Z seb $
+#
+#  An example that show how to use a  Yocto-MaxiPowerRelay
+#
+#  You can find more information on our web site:
+#   Yocto-MaxiPowerRelay documentation:
+#      https://www.yoctopuce.com/EN/products/yocto-maxipowerrelay/doc.html
+#   Python API Reference:
+#      https://www.yoctopuce.com/EN/doc/reference/yoctolib-python-EN.html
+#
+# *********************************************************************
+
 #!/usr/bin/python
 # -*- coding: utf-8 -*-
 import os, sys
 # add ../../Sources to the PYTHONPATH
-sys.path.append(os.path.join("..","..","Sources"))
 sys.path.append(os.path.join("..", "..", "Sources"))
+
 from yoctopuce.yocto_api import *
 from yoctopuce.yocto_relay import *
 
@@ -55,4 +69,5 @@ if relay.isOnline():
     else:
         relay.set_output(YRelay.OUTPUT_OFF)
 else:
-    die('Module not connected')
\ No newline at end of file
+    die('Module not connected')
+YAPI.FreeAPI()
\ No newline at end of file
