@@ -1,8 +1,23 @@
+# ********************************************************************
+#
+#  $Id: helloworld.py 32630 2018-10-10 14:11:07Z seb $
+#
+#  An example that show how to use a  Yocto-PowerRelay
+#
+#  You can find more information on our web site:
+#   Yocto-PowerRelay documentation:
+#      https://www.yoctopuce.com/EN/products/yocto-powerrelay/doc.html
+#   Python API Reference:
+#      https://www.yoctopuce.com/EN/doc/reference/yoctolib-python-EN.html
+#
+# *********************************************************************
+
 #!/usr/bin/python
 # -*- coding: utf-8 -*-
 import os, sys
 # add ../../Sources to the PYTHONPATH
 sys.path.append(os.path.join("..", "..", "Sources"))
+
 from yoctopuce.yocto_api import *
 from yoctopuce.yocto_relay import *
 
@@ -39,7 +54,7 @@ if target == 'ANY':
     if relay is None:
         die('no device connected')
 else:
-    relay = YRelay.FindRelay(target + "relay1")
+    relay = YRelay.FindRelay(target + ".relay1")
 
 if not (relay.isOnline()):
     die('device not connected')
@@ -48,3 +63,4 @@ if state == 'A':
     relay.set_state(YRelay.STATE_A)
 else:
     relay.set_state(YRelay.STATE_B)
+YAPI.FreeAPI()
