@@ -1,9 +1,21 @@
+# ********************************************************************
+#
+#  $Id: svn_id $
+#
+#  Doc-ModuleControl example
+#
+#  You can find more information on our web site:
+#   Python API Reference:
+#      https://www.yoctopuce.com/EN/doc/reference/yoctolib-python-EN.html
+#
+# *********************************************************************
+
 #!/usr/bin/python
 # -*- coding: utf-8 -*-
-import os,sys
-# add ../../Sources to the PYTHONPATH
-sys.path.append(os.path.join("..","..","Sources"))
 import os, sys
+# add ../../Sources to the PYTHONPATH
+sys.path.append(os.path.join("..", "..", "Sources"))
+
 from yoctopuce.yocto_api import *
 
 
@@ -39,6 +51,4 @@ if m.isOnline():
     print("logs:\n" + m.get_lastLogs())
 else:
     print(sys.argv[1] + " not connected (check identification and USB cable)")
-
-
-
+YAPI.FreeAPI()
