@@ -1,9 +1,21 @@
+# ********************************************************************
+#
+#  $Id: svn_id $
+#
+#  Doc-SaveSettings example
+#
+#  You can find more information on our web site:
+#   Python API Reference:
+#      https://www.yoctopuce.com/EN/doc/reference/yoctolib-python-EN.html
+#
+# *********************************************************************
+
 #!/usr/bin/python
 # -*- coding: utf-8 -*-
-import os,sys
-# add ../../Sources to the PYTHONPATH
-sys.path.append(os.path.join("..","..","Sources"))
 import os, sys
+# add ../../Sources to the PYTHONPATH
+sys.path.append(os.path.join("..", "..", "Sources"))
+
 from yoctopuce.yocto_api import *
 
 
@@ -28,4 +40,4 @@ if m.isOnline():
     print("Module: serial= " + m.get_serialNumber() + " / name= " + m.get_logicalName())
 else:
     sys.exit("not connected (check identification and USB cable")
-
+YAPI.FreeAPI()
\ No newline at end of file
