@@ -1,41 +1,42 @@
-#*********************************************************************
-#*
-#* $Id: yocto_bluetoothlink.py 23243 2016-02-23 14:13:12Z seb $
-#*
-#* Implements yFindBluetoothLink(), the high-level API for BluetoothLink functions
-#*
-#* - - - - - - - - - License information: - - - - - - - - - 
-#*
-#*  Copyright (C) 2011 and beyond by Yoctopuce Sarl, Switzerland.
-#*
-#*  Yoctopuce Sarl (hereafter Licensor) grants to you a perpetual
-#*  non-exclusive license to use, modify, copy and integrate this
-#*  file into your software for the sole purpose of interfacing
-#*  with Yoctopuce products.
-#*
-#*  You may reproduce and distribute copies of this file in
-#*  source or object form, as long as the sole purpose of this
-#*  code is to interface with Yoctopuce products. You must retain
-#*  this notice in the distributed source file.
-#*
-#*  You should refer to Yoctopuce General Terms and Conditions
-#*  for additional information regarding your rights and
-#*  obligations.
-#*
-#*  THE SOFTWARE AND DOCUMENTATION ARE PROVIDED 'AS IS' WITHOUT
-#*  WARRANTY OF ANY KIND, EITHER EXPRESS OR IMPLIED, INCLUDING 
-#*  WITHOUT LIMITATION, ANY WARRANTY OF MERCHANTABILITY, FITNESS
-#*  FOR A PARTICULAR PURPOSE, TITLE AND NON-INFRINGEMENT. IN NO
-#*  EVENT SHALL LICENSOR BE LIABLE FOR ANY INCIDENTAL, SPECIAL,
-#*  INDIRECT OR CONSEQUENTIAL DAMAGES, LOST PROFITS OR LOST DATA,
-#*  COST OF PROCUREMENT OF SUBSTITUTE GOODS, TECHNOLOGY OR
-#*  SERVICES, ANY CLAIMS BY THIRD PARTIES (INCLUDING BUT NOT
-#*  LIMITED TO ANY DEFENSE THEREOF), ANY CLAIMS FOR INDEMNITY OR
-#*  CONTRIBUTION, OR OTHER SIMILAR COSTS, WHETHER ASSERTED ON THE
-#*  BASIS OF CONTRACT, TORT (INCLUDING NEGLIGENCE), BREACH OF
-#*  WARRANTY, OR OTHERWISE.
-#*
-#*********************************************************************/
+# -*- coding: utf-8 -*-
+# ********************************************************************
+#
+#  $Id: yocto_bluetoothlink.py 38899 2019-12-20 17:21:03Z mvuilleu $
+#
+#  Implements yFindBluetoothLink(), the high-level API for BluetoothLink functions
+#
+#  - - - - - - - - - License information: - - - - - - - - -
+#
+#  Copyright (C) 2011 and beyond by Yoctopuce Sarl, Switzerland.
+#
+#  Yoctopuce Sarl (hereafter Licensor) grants to you a perpetual
+#  non-exclusive license to use, modify, copy and integrate this
+#  file into your software for the sole purpose of interfacing
+#  with Yoctopuce products.
+#
+#  You may reproduce and distribute copies of this file in
+#  source or object form, as long as the sole purpose of this
+#  code is to interface with Yoctopuce products. You must retain
+#  this notice in the distributed source file.
+#
+#  You should refer to Yoctopuce General Terms and Conditions
+#  for additional information regarding your rights and
+#  obligations.
+#
+#  THE SOFTWARE AND DOCUMENTATION ARE PROVIDED 'AS IS' WITHOUT
+#  WARRANTY OF ANY KIND, EITHER EXPRESS OR IMPLIED, INCLUDING
+#  WITHOUT LIMITATION, ANY WARRANTY OF MERCHANTABILITY, FITNESS
+#  FOR A PARTICULAR PURPOSE, TITLE AND NON-INFRINGEMENT. IN NO
+#  EVENT SHALL LICENSOR BE LIABLE FOR ANY INCIDENTAL, SPECIAL,
+#  INDIRECT OR CONSEQUENTIAL DAMAGES, LOST PROFITS OR LOST DATA,
+#  COST OF PROCUREMENT OF SUBSTITUTE GOODS, TECHNOLOGY OR
+#  SERVICES, ANY CLAIMS BY THIRD PARTIES (INCLUDING BUT NOT
+#  LIMITED TO ANY DEFENSE THEREOF), ANY CLAIMS FOR INDEMNITY OR
+#  CONTRIBUTION, OR OTHER SIMILAR COSTS, WHETHER ASSERTED ON THE
+#  BASIS OF CONTRACT, TORT (INCLUDING NEGLIGENCE), BREACH OF
+#  WARRANTY, OR OTHERWISE.
+#
+# *********************************************************************
 
 
 __docformat__ = 'restructuredtext en'
@@ -46,15 +47,17 @@ from yoctopuce.yocto_api import *
 #noinspection PyProtectedMember
 class YBluetoothLink(YFunction):
     """
-    BluetoothLink function provides control over bluetooth link
-    and status for devices that are bluetooth-enabled.
+    BluetoothLink function provides control over Bluetooth link
+    and status for devices that are Bluetooth-enabled.
 
     """
-#--- (end of YBluetoothLink class start)
+    #--- (end of YBluetoothLink class start)
     #--- (YBluetoothLink return codes)
     #--- (end of YBluetoothLink return codes)
     #--- (YBluetoothLink dlldef)
     #--- (end of YBluetoothLink dlldef)
+    #--- (YBluetoothLink yapiwrapper)
+    #--- (end of YBluetoothLink yapiwrapper)
     #--- (YBluetoothLink definitions)
     OWNADDRESS_INVALID = YAPI.INVALID_STRING
     PAIRINGPIN_INVALID = YAPI.INVALID_STRING
@@ -94,38 +97,28 @@ class YBluetoothLink(YFunction):
         #--- (end of YBluetoothLink attributes)
 
     #--- (YBluetoothLink implementation)
-    def _parseAttr(self, member):
-        if member.name == "ownAddress":
-            self._ownAddress = member.svalue
-            return 1
-        if member.name == "pairingPin":
-            self._pairingPin = member.svalue
-            return 1
-        if member.name == "remoteAddress":
-            self._remoteAddress = member.svalue
-            return 1
-        if member.name == "remoteName":
-            self._remoteName = member.svalue
-            return 1
-        if member.name == "mute":
-            self._mute = member.ivalue
-            return 1
-        if member.name == "preAmplifier":
-            self._preAmplifier = member.ivalue
-            return 1
-        if member.name == "volume":
-            self._volume = member.ivalue
-            return 1
-        if member.name == "linkState":
-            self._linkState = member.ivalue
-            return 1
-        if member.name == "linkQuality":
-            self._linkQuality = member.ivalue
-            return 1
-        if member.name == "command":
-            self._command = member.svalue
-            return 1
-        super(YBluetoothLink, self)._parseAttr(member)
+    def _parseAttr(self, json_val):
+        if json_val.has("ownAddress"):
+            self._ownAddress = json_val.getString("ownAddress")
+        if json_val.has("pairingPin"):
+            self._pairingPin = json_val.getString("pairingPin")
+        if json_val.has("remoteAddress"):
+            self._remoteAddress = json_val.getString("remoteAddress")
+        if json_val.has("remoteName"):
+            self._remoteName = json_val.getString("remoteName")
+        if json_val.has("mute"):
+            self._mute = (json_val.getInt("mute") > 0 if 1 else 0)
+        if json_val.has("preAmplifier"):
+            self._preAmplifier = json_val.getInt("preAmplifier")
+        if json_val.has("volume"):
+            self._volume = json_val.getInt("volume")
+        if json_val.has("linkState"):
+            self._linkState = json_val.getInt("linkState")
+        if json_val.has("linkQuality"):
+            self._linkQuality = json_val.getInt("linkQuality")
+        if json_val.has("command"):
+            self._command = json_val.getString("command")
+        super(YBluetoothLink, self)._parseAttr(json_val)
 
     def get_ownAddress(self):
         """
@@ -136,10 +129,12 @@ class YBluetoothLink(YFunction):
 
         On failure, throws an exception or returns YBluetoothLink.OWNADDRESS_INVALID.
         """
+        # res
         if self._cacheExpiration <= YAPI.GetTickCount():
-            if self.load(YAPI.DefaultCacheValidity) != YAPI.SUCCESS:
+            if self.load(YAPI._yapiContext.GetCacheValidity()) != YAPI.SUCCESS:
                 return YBluetoothLink.OWNADDRESS_INVALID
-        return self._ownAddress
+        res = self._ownAddress
+        return res
 
     def get_pairingPin(self):
         """
@@ -153,10 +148,12 @@ class YBluetoothLink(YFunction):
 
         On failure, throws an exception or returns YBluetoothLink.PAIRINGPIN_INVALID.
         """
+        # res
         if self._cacheExpiration <= YAPI.GetTickCount():
-            if self.load(YAPI.DefaultCacheValidity) != YAPI.SUCCESS:
+            if self.load(YAPI._yapiContext.GetCacheValidity()) != YAPI.SUCCESS:
                 return YBluetoothLink.PAIRINGPIN_INVALID
-        return self._pairingPin
+        res = self._pairingPin
+        return res
 
     def set_pairingPin(self, newval):
         """
@@ -181,14 +178,18 @@ class YBluetoothLink(YFunction):
 
         On failure, throws an exception or returns YBluetoothLink.REMOTEADDRESS_INVALID.
         """
+        # res
         if self._cacheExpiration <= YAPI.GetTickCount():
-            if self.load(YAPI.DefaultCacheValidity) != YAPI.SUCCESS:
+            if self.load(YAPI._yapiContext.GetCacheValidity()) != YAPI.SUCCESS:
                 return YBluetoothLink.REMOTEADDRESS_INVALID
-        return self._remoteAddress
+        res = self._remoteAddress
+        return res
 
     def set_remoteAddress(self, newval):
         """
         Changes the MAC-48 address defining which remote device to connect to.
+        Remember to call the saveToFlash()
+        method of the module if the modification must be kept.
 
         @param newval : a string corresponding to the MAC-48 address defining which remote device to connect to
 
@@ -207,10 +208,12 @@ class YBluetoothLink(YFunction):
 
         On failure, throws an exception or returns YBluetoothLink.REMOTENAME_INVALID.
         """
+        # res
         if self._cacheExpiration <= YAPI.GetTickCount():
-            if self.load(YAPI.DefaultCacheValidity) != YAPI.SUCCESS:
+            if self.load(YAPI._yapiContext.GetCacheValidity()) != YAPI.SUCCESS:
                 return YBluetoothLink.REMOTENAME_INVALID
-        return self._remoteName
+        res = self._remoteName
+        return res
 
     def get_mute(self):
         """
@@ -221,10 +224,12 @@ class YBluetoothLink(YFunction):
 
         On failure, throws an exception or returns YBluetoothLink.MUTE_INVALID.
         """
+        # res
         if self._cacheExpiration <= YAPI.GetTickCount():
-            if self.load(YAPI.DefaultCacheValidity) != YAPI.SUCCESS:
+            if self.load(YAPI._yapiContext.GetCacheValidity()) != YAPI.SUCCESS:
                 return YBluetoothLink.MUTE_INVALID
-        return self._mute
+        res = self._mute
+        return res
 
     def set_mute(self, newval):
         """
@@ -249,14 +254,18 @@ class YBluetoothLink(YFunction):
 
         On failure, throws an exception or returns YBluetoothLink.PREAMPLIFIER_INVALID.
         """
+        # res
         if self._cacheExpiration <= YAPI.GetTickCount():
-            if self.load(YAPI.DefaultCacheValidity) != YAPI.SUCCESS:
+            if self.load(YAPI._yapiContext.GetCacheValidity()) != YAPI.SUCCESS:
                 return YBluetoothLink.PREAMPLIFIER_INVALID
-        return self._preAmplifier
+        res = self._preAmplifier
+        return res
 
     def set_preAmplifier(self, newval):
         """
         Changes the audio pre-amplifier volume, in per cents.
+        Remember to call the saveToFlash()
+        method of the module if the modification must be kept.
 
         @param newval : an integer corresponding to the audio pre-amplifier volume, in per cents
 
@@ -275,10 +284,12 @@ class YBluetoothLink(YFunction):
 
         On failure, throws an exception or returns YBluetoothLink.VOLUME_INVALID.
         """
+        # res
         if self._cacheExpiration <= YAPI.GetTickCount():
-            if self.load(YAPI.DefaultCacheValidity) != YAPI.SUCCESS:
+            if self.load(YAPI._yapiContext.GetCacheValidity()) != YAPI.SUCCESS:
                 return YBluetoothLink.VOLUME_INVALID
-        return self._volume
+        res = self._volume
+        return res
 
     def set_volume(self, newval):
         """
@@ -303,10 +314,12 @@ class YBluetoothLink(YFunction):
 
         On failure, throws an exception or returns YBluetoothLink.LINKSTATE_INVALID.
         """
+        # res
         if self._cacheExpiration <= YAPI.GetTickCount():
-            if self.load(YAPI.DefaultCacheValidity) != YAPI.SUCCESS:
+            if self.load(YAPI._yapiContext.GetCacheValidity()) != YAPI.SUCCESS:
                 return YBluetoothLink.LINKSTATE_INVALID
-        return self._linkState
+        res = self._linkState
+        return res
 
     def get_linkQuality(self):
         """
@@ -317,16 +330,20 @@ class YBluetoothLink(YFunction):
 
         On failure, throws an exception or returns YBluetoothLink.LINKQUALITY_INVALID.
         """
+        # res
         if self._cacheExpiration <= YAPI.GetTickCount():
-            if self.load(YAPI.DefaultCacheValidity) != YAPI.SUCCESS:
+            if self.load(YAPI._yapiContext.GetCacheValidity()) != YAPI.SUCCESS:
                 return YBluetoothLink.LINKQUALITY_INVALID
-        return self._linkQuality
+        res = self._linkQuality
+        return res
 
     def get_command(self):
+        # res
         if self._cacheExpiration <= YAPI.GetTickCount():
-            if self.load(YAPI.DefaultCacheValidity) != YAPI.SUCCESS:
+            if self.load(YAPI._yapiContext.GetCacheValidity()) != YAPI.SUCCESS:
                 return YBluetoothLink.COMMAND_INVALID
-        return self._command
+        res = self._command
+        return res
 
     def set_command(self, newval):
         rest_val = newval
@@ -335,7 +352,7 @@ class YBluetoothLink(YFunction):
     @staticmethod
     def FindBluetoothLink(func):
         """
-        Retrieves a cellular interface for a given identifier.
+        Retrieves a Bluetooth sound controller for a given identifier.
         The identifier can be specified using several formats:
         <ul>
         <li>FunctionLogicalName</li>
@@ -345,17 +362,22 @@ class YBluetoothLink(YFunction):
         <li>ModuleLogicalName.FunctionLogicalName</li>
         </ul>
 
-        This function does not require that the cellular interface is online at the time
+        This function does not require that the Bluetooth sound controller is online at the time
         it is invoked. The returned object is nevertheless valid.
-        Use the method YBluetoothLink.isOnline() to test if the cellular interface is
+        Use the method YBluetoothLink.isOnline() to test if the Bluetooth sound controller is
         indeed online at a given time. In case of ambiguity when looking for
-        a cellular interface by logical name, no error is notified: the first instance
+        a Bluetooth sound controller by logical name, no error is notified: the first instance
         found is returned. The search is performed first by hardware name,
         then by logical name.
 
-        @param func : a string that uniquely characterizes the cellular interface
+        If a call to this object's is_online() method returns FALSE although
+        you are certain that the matching device is plugged, make sure that you did
+        call registerHub() at application initialization time.
 
-        @return a YBluetoothLink object allowing you to drive the cellular interface.
+        @param func : a string that uniquely characterizes the Bluetooth sound controller, for instance
+                MyDevice.bluetoothLink1.
+
+        @return a YBluetoothLink object allowing you to drive the Bluetooth sound controller.
         """
         # obj
         obj = YFunction._FindFromCache("BluetoothLink", func)
@@ -386,11 +408,14 @@ class YBluetoothLink(YFunction):
 
     def nextBluetoothLink(self):
         """
-        Continues the enumeration of cellular interfaces started using yFirstBluetoothLink().
+        Continues the enumeration of Bluetooth sound controllers started using yFirstBluetoothLink().
+        Caution: You can't make any assumption about the returned Bluetooth sound controllers order.
+        If you want to find a specific a Bluetooth sound controller, use BluetoothLink.findBluetoothLink()
+        and a hardwareID or a logical name.
 
         @return a pointer to a YBluetoothLink object, corresponding to
-                a cellular interface currently online, or a None pointer
-                if there are no more cellular interfaces to enumerate.
+                a Bluetooth sound controller currently online, or a None pointer
+                if there are no more Bluetooth sound controllers to enumerate.
         """
         hwidRef = YRefParam()
         if YAPI.YISERR(self._nextFunction(hwidRef)):
@@ -401,17 +426,17 @@ class YBluetoothLink(YFunction):
 
 #--- (end of YBluetoothLink implementation)
 
-#--- (BluetoothLink functions)
+#--- (YBluetoothLink functions)
 
     @staticmethod
     def FirstBluetoothLink():
         """
-        Starts the enumeration of cellular interfaces currently accessible.
+        Starts the enumeration of Bluetooth sound controllers currently accessible.
         Use the method YBluetoothLink.nextBluetoothLink() to iterate on
-        next cellular interfaces.
+        next Bluetooth sound controllers.
 
         @return a pointer to a YBluetoothLink object, corresponding to
-                the first cellular interface currently online, or a None pointer
+                the first Bluetooth sound controller currently online, or a None pointer
                 if there are none.
         """
         devRef = YRefParam()
@@ -435,4 +460,4 @@ class YBluetoothLink(YFunction):
 
         return YBluetoothLink.FindBluetoothLink(serialRef.value + "." + funcIdRef.value)
 
-#--- (end of BluetoothLink functions)
+#--- (end of YBluetoothLink functions)
