@@ -1,41 +1,42 @@
-#*********************************************************************
-#*
-#* $Id: yocto_colorled.py 24475 2016-05-12 14:03:35Z mvuilleu $
-#*
-#* Implements yFindColorLed(), the high-level API for ColorLed functions
-#*
-#* - - - - - - - - - License information: - - - - - - - - - 
-#*
-#*  Copyright (C) 2011 and beyond by Yoctopuce Sarl, Switzerland.
-#*
-#*  Yoctopuce Sarl (hereafter Licensor) grants to you a perpetual
-#*  non-exclusive license to use, modify, copy and integrate this
-#*  file into your software for the sole purpose of interfacing
-#*  with Yoctopuce products.
-#*
-#*  You may reproduce and distribute copies of this file in
-#*  source or object form, as long as the sole purpose of this
-#*  code is to interface with Yoctopuce products. You must retain
-#*  this notice in the distributed source file.
-#*
-#*  You should refer to Yoctopuce General Terms and Conditions
-#*  for additional information regarding your rights and
-#*  obligations.
-#*
-#*  THE SOFTWARE AND DOCUMENTATION ARE PROVIDED 'AS IS' WITHOUT
-#*  WARRANTY OF ANY KIND, EITHER EXPRESS OR IMPLIED, INCLUDING 
-#*  WITHOUT LIMITATION, ANY WARRANTY OF MERCHANTABILITY, FITNESS
-#*  FOR A PARTICULAR PURPOSE, TITLE AND NON-INFRINGEMENT. IN NO
-#*  EVENT SHALL LICENSOR BE LIABLE FOR ANY INCIDENTAL, SPECIAL,
-#*  INDIRECT OR CONSEQUENTIAL DAMAGES, LOST PROFITS OR LOST DATA,
-#*  COST OF PROCUREMENT OF SUBSTITUTE GOODS, TECHNOLOGY OR
-#*  SERVICES, ANY CLAIMS BY THIRD PARTIES (INCLUDING BUT NOT
-#*  LIMITED TO ANY DEFENSE THEREOF), ANY CLAIMS FOR INDEMNITY OR
-#*  CONTRIBUTION, OR OTHER SIMILAR COSTS, WHETHER ASSERTED ON THE
-#*  BASIS OF CONTRACT, TORT (INCLUDING NEGLIGENCE), BREACH OF
-#*  WARRANTY, OR OTHERWISE.
-#*
-#*********************************************************************/
+# -*- coding: utf-8 -*-
+# ********************************************************************
+#
+#  $Id: yocto_colorled.py 38899 2019-12-20 17:21:03Z mvuilleu $
+#
+#  Implements yFindColorLed(), the high-level API for ColorLed functions
+#
+#  - - - - - - - - - License information: - - - - - - - - -
+#
+#  Copyright (C) 2011 and beyond by Yoctopuce Sarl, Switzerland.
+#
+#  Yoctopuce Sarl (hereafter Licensor) grants to you a perpetual
+#  non-exclusive license to use, modify, copy and integrate this
+#  file into your software for the sole purpose of interfacing
+#  with Yoctopuce products.
+#
+#  You may reproduce and distribute copies of this file in
+#  source or object form, as long as the sole purpose of this
+#  code is to interface with Yoctopuce products. You must retain
+#  this notice in the distributed source file.
+#
+#  You should refer to Yoctopuce General Terms and Conditions
+#  for additional information regarding your rights and
+#  obligations.
+#
+#  THE SOFTWARE AND DOCUMENTATION ARE PROVIDED 'AS IS' WITHOUT
+#  WARRANTY OF ANY KIND, EITHER EXPRESS OR IMPLIED, INCLUDING
+#  WITHOUT LIMITATION, ANY WARRANTY OF MERCHANTABILITY, FITNESS
+#  FOR A PARTICULAR PURPOSE, TITLE AND NON-INFRINGEMENT. IN NO
+#  EVENT SHALL LICENSOR BE LIABLE FOR ANY INCIDENTAL, SPECIAL,
+#  INDIRECT OR CONSEQUENTIAL DAMAGES, LOST PROFITS OR LOST DATA,
+#  COST OF PROCUREMENT OF SUBSTITUTE GOODS, TECHNOLOGY OR
+#  SERVICES, ANY CLAIMS BY THIRD PARTIES (INCLUDING BUT NOT
+#  LIMITED TO ANY DEFENSE THEREOF), ANY CLAIMS FOR INDEMNITY OR
+#  CONTRIBUTION, OR OTHER SIMILAR COSTS, WHETHER ASSERTED ON THE
+#  BASIS OF CONTRACT, TORT (INCLUDING NEGLIGENCE), BREACH OF
+#  WARRANTY, OR OTHERWISE.
+#
+# *********************************************************************
 
 
 __docformat__ = 'restructuredtext en'
@@ -46,19 +47,21 @@ from yoctopuce.yocto_api import *
 #noinspection PyProtectedMember
 class YColorLed(YFunction):
     """
-    The Yoctopuce application programming interface
-    allows you to drive a color LED using RGB coordinates as well as HSL coordinates.
+    The ColorLed class allows you to drive a color LED.
+    The color can be specified using RGB coordinates as well as HSL coordinates.
     The module performs all conversions form RGB to HSL automatically. It is then
     self-evident to turn on a LED with a given hue and to progressively vary its
     saturation or lightness. If needed, you can find more information on the
     difference between RGB and HSL in the section following this one.
 
     """
-#--- (end of YColorLed class start)
+    #--- (end of YColorLed class start)
     #--- (YColorLed return codes)
     #--- (end of YColorLed return codes)
     #--- (YColorLed dlldef)
     #--- (end of YColorLed dlldef)
+    #--- (YColorLed yapiwrapper)
+    #--- (end of YColorLed yapiwrapper)
     #--- (YColorLed definitions)
     RGBCOLOR_INVALID = YAPI.INVALID_UINT
     HSLCOLOR_INVALID = YAPI.INVALID_UINT
@@ -88,53 +91,40 @@ class YColorLed(YFunction):
         #--- (end of YColorLed attributes)
 
     #--- (YColorLed implementation)
-    def _parseAttr(self, member):
-        if member.name == "rgbColor":
-            self._rgbColor = member.ivalue
-            return 1
-        if member.name == "hslColor":
-            self._hslColor = member.ivalue
-            return 1
-        if member.name == "rgbMove":
-            if member.recordtype != YAPI.TJSONRECORDTYPE.JSON_STRUCT:
-                self._rgbMove = -1
+    def _parseAttr(self, json_val):
+        if json_val.has("rgbColor"):
+            self._rgbColor = json_val.getInt("rgbColor")
+        if json_val.has("hslColor"):
+            self._hslColor = json_val.getInt("hslColor")
+        if json_val.has("rgbMove"):
+            subjson = json_val.getYJSONObject("rgbMove");
             self._rgbMove = {"moving": None, "target": None, "ms": None}
-            for submemb in member.members:
-                if submemb.name == "moving":
-                    self._rgbMove["moving"] = submemb.ivalue
-                elif submemb.name == "target":
-                    self._rgbMove["target"] = submemb.ivalue
-                elif submemb.name == "ms":
-                    self._rgbMove["ms"] = submemb.ivalue
-            return 1
-        if member.name == "hslMove":
-            if member.recordtype != YAPI.TJSONRECORDTYPE.JSON_STRUCT:
-                self._hslMove = -1
+            if subjson.has("moving"):
+                self._rgbMove["moving"] = subjson.getInt("moving")
+            if subjson.has("target"):
+                self._rgbMove["target"] = subjson.getInt("target")
+            if subjson.has("ms"):
+                self._rgbMove["ms"] = subjson.getInt("ms")
+        if json_val.has("hslMove"):
+            subjson = json_val.getYJSONObject("hslMove");
             self._hslMove = {"moving": None, "target": None, "ms": None}
-            for submemb in member.members:
-                if submemb.name == "moving":
-                    self._hslMove["moving"] = submemb.ivalue
-                elif submemb.name == "target":
-                    self._hslMove["target"] = submemb.ivalue
-                elif submemb.name == "ms":
-                    self._hslMove["ms"] = submemb.ivalue
-            return 1
-        if member.name == "rgbColorAtPowerOn":
-            self._rgbColorAtPowerOn = member.ivalue
-            return 1
-        if member.name == "blinkSeqSize":
-            self._blinkSeqSize = member.ivalue
-            return 1
-        if member.name == "blinkSeqMaxSize":
-            self._blinkSeqMaxSize = member.ivalue
-            return 1
-        if member.name == "blinkSeqSignature":
-            self._blinkSeqSignature = member.ivalue
-            return 1
-        if member.name == "command":
-            self._command = member.svalue
-            return 1
-        super(YColorLed, self)._parseAttr(member)
+            if subjson.has("moving"):
+                self._hslMove["moving"] = subjson.getInt("moving")
+            if subjson.has("target"):
+                self._hslMove["target"] = subjson.getInt("target")
+            if subjson.has("ms"):
+                self._hslMove["ms"] = subjson.getInt("ms")
+        if json_val.has("rgbColorAtPowerOn"):
+            self._rgbColorAtPowerOn = json_val.getInt("rgbColorAtPowerOn")
+        if json_val.has("blinkSeqSize"):
+            self._blinkSeqSize = json_val.getInt("blinkSeqSize")
+        if json_val.has("blinkSeqMaxSize"):
+            self._blinkSeqMaxSize = json_val.getInt("blinkSeqMaxSize")
+        if json_val.has("blinkSeqSignature"):
+            self._blinkSeqSignature = json_val.getInt("blinkSeqSignature")
+        if json_val.has("command"):
+            self._command = json_val.getString("command")
+        super(YColorLed, self)._parseAttr(json_val)
 
     def get_rgbColor(self):
         """
@@ -144,10 +134,12 @@ class YColorLed(YFunction):
 
         On failure, throws an exception or returns YColorLed.RGBCOLOR_INVALID.
         """
+        # res
         if self._cacheExpiration <= YAPI.GetTickCount():
-            if self.load(YAPI.DefaultCacheValidity) != YAPI.SUCCESS:
+            if self.load(YAPI._yapiContext.GetCacheValidity()) != YAPI.SUCCESS:
                 return YColorLed.RGBCOLOR_INVALID
-        return self._rgbColor
+        res = self._rgbColor
+        return res
 
     def set_rgbColor(self, newval):
         """
@@ -170,10 +162,12 @@ class YColorLed(YFunction):
 
         On failure, throws an exception or returns YColorLed.HSLCOLOR_INVALID.
         """
+        # res
         if self._cacheExpiration <= YAPI.GetTickCount():
-            if self.load(YAPI.DefaultCacheValidity) != YAPI.SUCCESS:
+            if self.load(YAPI._yapiContext.GetCacheValidity()) != YAPI.SUCCESS:
                 return YColorLed.HSLCOLOR_INVALID
-        return self._hslColor
+        res = self._hslColor
+        return res
 
     def set_hslColor(self, newval):
         """
@@ -189,10 +183,12 @@ class YColorLed(YFunction):
         return self._setAttr("hslColor", rest_val)
 
     def get_rgbMove(self):
+        # res
         if self._cacheExpiration <= YAPI.GetTickCount():
-            if self.load(YAPI.DefaultCacheValidity) != YAPI.SUCCESS:
+            if self.load(YAPI._yapiContext.GetCacheValidity()) != YAPI.SUCCESS:
                 return YColorLed.RGBMOVE_INVALID
-        return self._rgbMove
+        res = self._rgbMove
+        return res
 
     def set_rgbMove(self, newval):
         rest_val = str(newval.target) + ":" + str(newval.ms)
@@ -213,10 +209,12 @@ class YColorLed(YFunction):
         return self._setAttr("rgbMove", rest_val)
 
     def get_hslMove(self):
+        # res
         if self._cacheExpiration <= YAPI.GetTickCount():
-            if self.load(YAPI.DefaultCacheValidity) != YAPI.SUCCESS:
+            if self.load(YAPI._yapiContext.GetCacheValidity()) != YAPI.SUCCESS:
                 return YColorLed.HSLMOVE_INVALID
-        return self._hslMove
+        res = self._hslMove
+        return res
 
     def set_hslMove(self, newval):
         rest_val = str(newval.target) + ":" + str(newval.ms)
@@ -244,16 +242,20 @@ class YColorLed(YFunction):
 
         On failure, throws an exception or returns YColorLed.RGBCOLORATPOWERON_INVALID.
         """
+        # res
         if self._cacheExpiration <= YAPI.GetTickCount():
-            if self.load(YAPI.DefaultCacheValidity) != YAPI.SUCCESS:
+            if self.load(YAPI._yapiContext.GetCacheValidity()) != YAPI.SUCCESS:
                 return YColorLed.RGBCOLORATPOWERON_INVALID
-        return self._rgbColorAtPowerOn
+        res = self._rgbColorAtPowerOn
+        return res
 
     def set_rgbColorAtPowerOn(self, newval):
         """
-        Changes the color that the LED will display by default when the module is turned on.
+        Changes the color that the LED displays by default when the module is turned on.
+        Remember to call the saveToFlash()
+        method of the module if the modification must be kept.
 
-        @param newval : an integer corresponding to the color that the LED will display by default when the
+        @param newval : an integer corresponding to the color that the LED displays by default when the
         module is turned on
 
         @return YAPI.SUCCESS if the call succeeds.
@@ -271,10 +273,12 @@ class YColorLed(YFunction):
 
         On failure, throws an exception or returns YColorLed.BLINKSEQSIZE_INVALID.
         """
+        # res
         if self._cacheExpiration <= YAPI.GetTickCount():
-            if self.load(YAPI.DefaultCacheValidity) != YAPI.SUCCESS:
+            if self.load(YAPI._yapiContext.GetCacheValidity()) != YAPI.SUCCESS:
                 return YColorLed.BLINKSEQSIZE_INVALID
-        return self._blinkSeqSize
+        res = self._blinkSeqSize
+        return res
 
     def get_blinkSeqMaxSize(self):
         """
@@ -284,10 +288,12 @@ class YColorLed(YFunction):
 
         On failure, throws an exception or returns YColorLed.BLINKSEQMAXSIZE_INVALID.
         """
-        if self._cacheExpiration == datetime.datetime.fromtimestamp(0):
-            if self.load(YAPI.DefaultCacheValidity) != YAPI.SUCCESS:
+        # res
+        if self._cacheExpiration == datetime.datetime.fromtimestamp(86400):
+            if self.load(YAPI._yapiContext.GetCacheValidity()) != YAPI.SUCCESS:
                 return YColorLed.BLINKSEQMAXSIZE_INVALID
-        return self._blinkSeqMaxSize
+        res = self._blinkSeqMaxSize
+        return res
 
     def get_blinkSeqSignature(self):
         """
@@ -300,16 +306,20 @@ class YColorLed(YFunction):
 
         On failure, throws an exception or returns YColorLed.BLINKSEQSIGNATURE_INVALID.
         """
+        # res
         if self._cacheExpiration <= YAPI.GetTickCount():
-            if self.load(YAPI.DefaultCacheValidity) != YAPI.SUCCESS:
+            if self.load(YAPI._yapiContext.GetCacheValidity()) != YAPI.SUCCESS:
                 return YColorLed.BLINKSEQSIGNATURE_INVALID
-        return self._blinkSeqSignature
+        res = self._blinkSeqSignature
+        return res
 
     def get_command(self):
+        # res
         if self._cacheExpiration <= YAPI.GetTickCount():
-            if self.load(YAPI.DefaultCacheValidity) != YAPI.SUCCESS:
+            if self.load(YAPI._yapiContext.GetCacheValidity()) != YAPI.SUCCESS:
                 return YColorLed.COMMAND_INVALID
-        return self._command
+        res = self._command
+        return res
 
     def set_command(self, newval):
         rest_val = newval
@@ -336,7 +346,12 @@ class YColorLed(YFunction):
         found is returned. The search is performed first by hardware name,
         then by logical name.
 
-        @param func : a string that uniquely characterizes the RGB LED
+        If a call to this object's is_online() method returns FALSE although
+        you are certain that the matching device is plugged, make sure that you did
+        call registerHub() at application initialization time.
+
+        @param func : a string that uniquely characterizes the RGB LED, for instance
+                YRGBLED2.colorLed1.
 
         @return a YColorLed object allowing you to drive the RGB LED.
         """
@@ -348,7 +363,6 @@ class YColorLed(YFunction):
         return obj
 
     def sendCommand(self, command):
-        # //may throw an exception
         return self.set_command(command)
 
     def addHslMoveToBlinkSeq(self, HSLcolor, msDelay):
@@ -356,7 +370,7 @@ class YColorLed(YFunction):
         Add a new transition to the blinking sequence, the move will
         be performed in the HSL space.
 
-        @param HSLcolor : desired HSL color when the traisntion is completed
+        @param HSLcolor : desired HSL color when the transition is completed
         @param msDelay : duration of the color transition, in milliseconds.
 
         @return YAPI.SUCCESS if the call succeeds.
@@ -409,6 +423,9 @@ class YColorLed(YFunction):
     def nextColorLed(self):
         """
         Continues the enumeration of RGB LEDs started using yFirstColorLed().
+        Caution: You can't make any assumption about the returned RGB LEDs order.
+        If you want to find a specific an RGB LED, use ColorLed.findColorLed()
+        and a hardwareID or a logical name.
 
         @return a pointer to a YColorLed object, corresponding to
                 an RGB LED currently online, or a None pointer
@@ -423,7 +440,7 @@ class YColorLed(YFunction):
 
 #--- (end of YColorLed implementation)
 
-#--- (ColorLed functions)
+#--- (YColorLed functions)
 
     @staticmethod
     def FirstColorLed():
@@ -457,4 +474,4 @@ class YColorLed(YFunction):
 
         return YColorLed.FindColorLed(serialRef.value + "." + funcIdRef.value)
 
-#--- (end of ColorLed functions)
+#--- (end of YColorLed functions)
