@@ -1,41 +1,42 @@
-#*********************************************************************
-#*
-#* $Id: yocto_genericsensor.py 23527 2016-03-18 21:49:19Z mvuilleu $
-#*
-#* Implements yFindGenericSensor(), the high-level API for GenericSensor functions
-#*
-#* - - - - - - - - - License information: - - - - - - - - - 
-#*
-#*  Copyright (C) 2011 and beyond by Yoctopuce Sarl, Switzerland.
-#*
-#*  Yoctopuce Sarl (hereafter Licensor) grants to you a perpetual
-#*  non-exclusive license to use, modify, copy and integrate this
-#*  file into your software for the sole purpose of interfacing
-#*  with Yoctopuce products.
-#*
-#*  You may reproduce and distribute copies of this file in
-#*  source or object form, as long as the sole purpose of this
-#*  code is to interface with Yoctopuce products. You must retain
-#*  this notice in the distributed source file.
-#*
-#*  You should refer to Yoctopuce General Terms and Conditions
-#*  for additional information regarding your rights and
-#*  obligations.
-#*
-#*  THE SOFTWARE AND DOCUMENTATION ARE PROVIDED 'AS IS' WITHOUT
-#*  WARRANTY OF ANY KIND, EITHER EXPRESS OR IMPLIED, INCLUDING 
-#*  WITHOUT LIMITATION, ANY WARRANTY OF MERCHANTABILITY, FITNESS
-#*  FOR A PARTICULAR PURPOSE, TITLE AND NON-INFRINGEMENT. IN NO
-#*  EVENT SHALL LICENSOR BE LIABLE FOR ANY INCIDENTAL, SPECIAL,
-#*  INDIRECT OR CONSEQUENTIAL DAMAGES, LOST PROFITS OR LOST DATA,
-#*  COST OF PROCUREMENT OF SUBSTITUTE GOODS, TECHNOLOGY OR
-#*  SERVICES, ANY CLAIMS BY THIRD PARTIES (INCLUDING BUT NOT
-#*  LIMITED TO ANY DEFENSE THEREOF), ANY CLAIMS FOR INDEMNITY OR
-#*  CONTRIBUTION, OR OTHER SIMILAR COSTS, WHETHER ASSERTED ON THE
-#*  BASIS OF CONTRACT, TORT (INCLUDING NEGLIGENCE), BREACH OF
-#*  WARRANTY, OR OTHERWISE.
-#*
-#*********************************************************************/
+# -*- coding: utf-8 -*-
+# ********************************************************************
+#
+#  $Id: yocto_genericsensor.py 38899 2019-12-20 17:21:03Z mvuilleu $
+#
+#  Implements yFindGenericSensor(), the high-level API for GenericSensor functions
+#
+#  - - - - - - - - - License information: - - - - - - - - -
+#
+#  Copyright (C) 2011 and beyond by Yoctopuce Sarl, Switzerland.
+#
+#  Yoctopuce Sarl (hereafter Licensor) grants to you a perpetual
+#  non-exclusive license to use, modify, copy and integrate this
+#  file into your software for the sole purpose of interfacing
+#  with Yoctopuce products.
+#
+#  You may reproduce and distribute copies of this file in
+#  source or object form, as long as the sole purpose of this
+#  code is to interface with Yoctopuce products. You must retain
+#  this notice in the distributed source file.
+#
+#  You should refer to Yoctopuce General Terms and Conditions
+#  for additional information regarding your rights and
+#  obligations.
+#
+#  THE SOFTWARE AND DOCUMENTATION ARE PROVIDED 'AS IS' WITHOUT
+#  WARRANTY OF ANY KIND, EITHER EXPRESS OR IMPLIED, INCLUDING
+#  WITHOUT LIMITATION, ANY WARRANTY OF MERCHANTABILITY, FITNESS
+#  FOR A PARTICULAR PURPOSE, TITLE AND NON-INFRINGEMENT. IN NO
+#  EVENT SHALL LICENSOR BE LIABLE FOR ANY INCIDENTAL, SPECIAL,
+#  INDIRECT OR CONSEQUENTIAL DAMAGES, LOST PROFITS OR LOST DATA,
+#  COST OF PROCUREMENT OF SUBSTITUTE GOODS, TECHNOLOGY OR
+#  SERVICES, ANY CLAIMS BY THIRD PARTIES (INCLUDING BUT NOT
+#  LIMITED TO ANY DEFENSE THEREOF), ANY CLAIMS FOR INDEMNITY OR
+#  CONTRIBUTION, OR OTHER SIMILAR COSTS, WHETHER ASSERTED ON THE
+#  BASIS OF CONTRACT, TORT (INCLUDING NEGLIGENCE), BREACH OF
+#  WARRANTY, OR OTHERWISE.
+#
+# *********************************************************************
 
 
 __docformat__ = 'restructuredtext en'
@@ -48,16 +49,18 @@ class YGenericSensor(YSensor):
     """
     The YGenericSensor class allows you to read and configure Yoctopuce signal
     transducers. It inherits from YSensor class the core functions to read measurements,
-    register callback functions, access to the autonomous datalogger.
+    to register callback functions, to access the autonomous datalogger.
     This class adds the ability to configure the automatic conversion between the
     measured signal and the corresponding engineering unit.
 
     """
-#--- (end of YGenericSensor class start)
+    #--- (end of YGenericSensor class start)
     #--- (YGenericSensor return codes)
     #--- (end of YGenericSensor return codes)
     #--- (YGenericSensor dlldef)
     #--- (end of YGenericSensor dlldef)
+    #--- (YGenericSensor yapiwrapper)
+    #--- (end of YGenericSensor yapiwrapper)
     #--- (YGenericSensor definitions)
     SIGNALVALUE_INVALID = YAPI.INVALID_DOUBLE
     SIGNALUNIT_INVALID = YAPI.INVALID_STRING
@@ -68,7 +71,11 @@ class YGenericSensor(YSensor):
     SIGNALSAMPLING_HIGH_RATE_FILTERED = 1
     SIGNALSAMPLING_LOW_NOISE = 2
     SIGNALSAMPLING_LOW_NOISE_FILTERED = 3
+    SIGNALSAMPLING_HIGHEST_RATE = 4
     SIGNALSAMPLING_INVALID = -1
+    ENABLED_FALSE = 0
+    ENABLED_TRUE = 1
+    ENABLED_INVALID = -1
     #--- (end of YGenericSensor definitions)
 
     def __init__(self, func):
@@ -82,29 +89,26 @@ class YGenericSensor(YSensor):
         self._valueRange = YGenericSensor.VALUERANGE_INVALID
         self._signalBias = YGenericSensor.SIGNALBIAS_INVALID
         self._signalSampling = YGenericSensor.SIGNALSAMPLING_INVALID
+        self._enabled = YGenericSensor.ENABLED_INVALID
         #--- (end of YGenericSensor attributes)
 
     #--- (YGenericSensor implementation)
-    def _parseAttr(self, member):
-        if member.name == "signalValue":
-            self._signalValue = round(member.ivalue * 1000.0 / 65536.0) / 1000.0
-            return 1
-        if member.name == "signalUnit":
-            self._signalUnit = member.svalue
-            return 1
-        if member.name == "signalRange":
-            self._signalRange = member.svalue
-            return 1
-        if member.name == "valueRange":
-            self._valueRange = member.svalue
-            return 1
-        if member.name == "signalBias":
-            self._signalBias = round(member.ivalue * 1000.0 / 65536.0) / 1000.0
-            return 1
-        if member.name == "signalSampling":
-            self._signalSampling = member.ivalue
-            return 1
-        super(YGenericSensor, self)._parseAttr(member)
+    def _parseAttr(self, json_val):
+        if json_val.has("signalValue"):
+            self._signalValue = round(json_val.getDouble("signalValue") * 1000.0 / 65536.0) / 1000.0
+        if json_val.has("signalUnit"):
+            self._signalUnit = json_val.getString("signalUnit")
+        if json_val.has("signalRange"):
+            self._signalRange = json_val.getString("signalRange")
+        if json_val.has("valueRange"):
+            self._valueRange = json_val.getString("valueRange")
+        if json_val.has("signalBias"):
+            self._signalBias = round(json_val.getDouble("signalBias") * 1000.0 / 65536.0) / 1000.0
+        if json_val.has("signalSampling"):
+            self._signalSampling = json_val.getInt("signalSampling")
+        if json_val.has("enabled"):
+            self._enabled = (json_val.getInt("enabled") > 0 if 1 else 0)
+        super(YGenericSensor, self)._parseAttr(json_val)
 
     def set_unit(self, newval):
         """
@@ -130,10 +134,12 @@ class YGenericSensor(YSensor):
 
         On failure, throws an exception or returns YGenericSensor.SIGNALVALUE_INVALID.
         """
+        # res
         if self._cacheExpiration <= YAPI.GetTickCount():
-            if self.load(YAPI.DefaultCacheValidity) != YAPI.SUCCESS:
+            if self.load(YAPI._yapiContext.GetCacheValidity()) != YAPI.SUCCESS:
                 return YGenericSensor.SIGNALVALUE_INVALID
-        return round(self._signalValue * 1000) / 1000
+        res = round(self._signalValue * 1000) / 1000
+        return res
 
     def get_signalUnit(self):
         """
@@ -143,29 +149,44 @@ class YGenericSensor(YSensor):
 
         On failure, throws an exception or returns YGenericSensor.SIGNALUNIT_INVALID.
         """
-        if self._cacheExpiration == datetime.datetime.fromtimestamp(0):
-            if self.load(YAPI.DefaultCacheValidity) != YAPI.SUCCESS:
+        # res
+        if self._cacheExpiration == datetime.datetime.fromtimestamp(86400):
+            if self.load(YAPI._yapiContext.GetCacheValidity()) != YAPI.SUCCESS:
                 return YGenericSensor.SIGNALUNIT_INVALID
-        return self._signalUnit
+        res = self._signalUnit
+        return res
 
     def get_signalRange(self):
         """
-        Returns the electric signal range used by the sensor.
+        Returns the input signal range used by the sensor.
 
-        @return a string corresponding to the electric signal range used by the sensor
+        @return a string corresponding to the input signal range used by the sensor
 
         On failure, throws an exception or returns YGenericSensor.SIGNALRANGE_INVALID.
         """
+        # res
         if self._cacheExpiration <= YAPI.GetTickCount():
-            if self.load(YAPI.DefaultCacheValidity) != YAPI.SUCCESS:
+            if self.load(YAPI._yapiContext.GetCacheValidity()) != YAPI.SUCCESS:
                 return YGenericSensor.SIGNALRANGE_INVALID
-        return self._signalRange
+        res = self._signalRange
+        return res
 
     def set_signalRange(self, newval):
         """
-        Changes the electric signal range used by the sensor.
+        Changes the input signal range used by the sensor.
+        When the input signal gets out of the planned range, the output value
+        will be set to an arbitrary large value, whose sign indicates the direction
+        of the range overrun.
 
-        @param newval : a string corresponding to the electric signal range used by the sensor
+        For a 4-20mA sensor, the default input signal range is "4...20".
+        For a 0-10V sensor, the default input signal range is "0.1...10".
+        For numeric communication interfaces, the default input signal range is
+        "-999999.999...999999.999".
+
+        Remember to call the saveToFlash()
+        method of the module if the modification must be kept.
+
+        @param newval : a string corresponding to the input signal range used by the sensor
 
         @return YAPI.SUCCESS if the call succeeds.
 
@@ -182,17 +203,26 @@ class YGenericSensor(YSensor):
 
         On failure, throws an exception or returns YGenericSensor.VALUERANGE_INVALID.
         """
+        # res
         if self._cacheExpiration <= YAPI.GetTickCount():
-            if self.load(YAPI.DefaultCacheValidity) != YAPI.SUCCESS:
+            if self.load(YAPI._yapiContext.GetCacheValidity()) != YAPI.SUCCESS:
                 return YGenericSensor.VALUERANGE_INVALID
-        return self._valueRange
+        res = self._valueRange
+        return res
 
     def set_valueRange(self, newval):
         """
-        Changes the physical value range measured by the sensor. As a side effect, the range modification may
-        automatically modify the display resolution.
+        Changes the output value range, corresponding to the physical value measured
+        by the sensor. The default output value range is the same as the input signal
+        range (1:1 mapping), but you can change it so that the function automatically
+        computes the physical value encoded by the input signal. Be aware that, as a
+        side effect, the range modification may automatically modify the display resolution.
 
-        @param newval : a string corresponding to the physical value range measured by the sensor
+        Remember to call the saveToFlash()
+        method of the module if the modification must be kept.
+
+        @param newval : a string corresponding to the output value range, corresponding to the physical value measured
+                by the sensor
 
         @return YAPI.SUCCESS if the call succeeds.
 
@@ -204,8 +234,10 @@ class YGenericSensor(YSensor):
     def set_signalBias(self, newval):
         """
         Changes the electric signal bias for zero shift adjustment.
-        If your electric signal reads positif when it should be zero, setup
+        If your electric signal reads positive when it should be zero, setup
         a positive signalBias of the same value to fix the zero shift.
+        Remember to call the saveToFlash()
+        method of the module if the modification must be kept.
 
         @param newval : a floating point number corresponding to the electric signal bias for zero shift adjustment
 
@@ -220,16 +252,18 @@ class YGenericSensor(YSensor):
         """
         Returns the electric signal bias for zero shift adjustment.
         A positive bias means that the signal is over-reporting the measure,
-        while a negative bias means that the signal is underreporting the measure.
+        while a negative bias means that the signal is under-reporting the measure.
 
         @return a floating point number corresponding to the electric signal bias for zero shift adjustment
 
         On failure, throws an exception or returns YGenericSensor.SIGNALBIAS_INVALID.
         """
+        # res
         if self._cacheExpiration <= YAPI.GetTickCount():
-            if self.load(YAPI.DefaultCacheValidity) != YAPI.SUCCESS:
+            if self.load(YAPI._yapiContext.GetCacheValidity()) != YAPI.SUCCESS:
                 return YGenericSensor.SIGNALBIAS_INVALID
-        return self._signalBias
+        res = self._signalBias
+        return res
 
     def get_signalSampling(self):
         """
@@ -241,15 +275,18 @@ class YGenericSensor(YSensor):
         to get measures as stable as possible when working on a noisy signal.
 
         @return a value among YGenericSensor.SIGNALSAMPLING_HIGH_RATE,
-        YGenericSensor.SIGNALSAMPLING_HIGH_RATE_FILTERED, YGenericSensor.SIGNALSAMPLING_LOW_NOISE and
-        YGenericSensor.SIGNALSAMPLING_LOW_NOISE_FILTERED corresponding to the electric signal sampling method to use
+        YGenericSensor.SIGNALSAMPLING_HIGH_RATE_FILTERED, YGenericSensor.SIGNALSAMPLING_LOW_NOISE,
+        YGenericSensor.SIGNALSAMPLING_LOW_NOISE_FILTERED and YGenericSensor.SIGNALSAMPLING_HIGHEST_RATE
+        corresponding to the electric signal sampling method to use
 
         On failure, throws an exception or returns YGenericSensor.SIGNALSAMPLING_INVALID.
         """
+        # res
         if self._cacheExpiration <= YAPI.GetTickCount():
-            if self.load(YAPI.DefaultCacheValidity) != YAPI.SUCCESS:
+            if self.load(YAPI._yapiContext.GetCacheValidity()) != YAPI.SUCCESS:
                 return YGenericSensor.SIGNALSAMPLING_INVALID
-        return self._signalSampling
+        res = self._signalSampling
+        return res
 
     def set_signalSampling(self, newval):
         """
@@ -259,10 +296,13 @@ class YGenericSensor(YSensor):
         The LOW_NOISE method uses a reduced acquisition frequency to reduce noise.
         The LOW_NOISE_FILTERED method combines a reduced frequency with the median filter
         to get measures as stable as possible when working on a noisy signal.
+        Remember to call the saveToFlash()
+        method of the module if the modification must be kept.
 
         @param newval : a value among YGenericSensor.SIGNALSAMPLING_HIGH_RATE,
-        YGenericSensor.SIGNALSAMPLING_HIGH_RATE_FILTERED, YGenericSensor.SIGNALSAMPLING_LOW_NOISE and
-        YGenericSensor.SIGNALSAMPLING_LOW_NOISE_FILTERED corresponding to the electric signal sampling method to use
+        YGenericSensor.SIGNALSAMPLING_HIGH_RATE_FILTERED, YGenericSensor.SIGNALSAMPLING_LOW_NOISE,
+        YGenericSensor.SIGNALSAMPLING_LOW_NOISE_FILTERED and YGenericSensor.SIGNALSAMPLING_HIGHEST_RATE
+        corresponding to the electric signal sampling method to use
 
         @return YAPI.SUCCESS if the call succeeds.
 
@@ -271,6 +311,40 @@ class YGenericSensor(YSensor):
         rest_val = str(newval)
         return self._setAttr("signalSampling", rest_val)
 
+    def get_enabled(self):
+        """
+        Returns the activation state of this input.
+
+        @return either YGenericSensor.ENABLED_FALSE or YGenericSensor.ENABLED_TRUE, according to the
+        activation state of this input
+
+        On failure, throws an exception or returns YGenericSensor.ENABLED_INVALID.
+        """
+        # res
+        if self._cacheExpiration <= YAPI.GetTickCount():
+            if self.load(YAPI._yapiContext.GetCacheValidity()) != YAPI.SUCCESS:
+                return YGenericSensor.ENABLED_INVALID
+        res = self._enabled
+        return res
+
+    def set_enabled(self, newval):
+        """
+        Changes the activation state of this input. When an input is disabled,
+        its value is no more updated. On some devices, disabling an input can
+        improve the refresh rate of the other active inputs.
+        Remember to call the saveToFlash()
+        method of the module if the modification must be kept.
+
+        @param newval : either YGenericSensor.ENABLED_FALSE or YGenericSensor.ENABLED_TRUE, according to
+        the activation state of this input
+
+        @return YAPI.SUCCESS if the call succeeds.
+
+        On failure, throws an exception or returns a negative error code.
+        """
+        rest_val = "1" if newval > 0 else "0"
+        return self._setAttr("enabled", rest_val)
+
     @staticmethod
     def FindGenericSensor(func):
         """
@@ -292,7 +366,12 @@ class YGenericSensor(YSensor):
         found is returned. The search is performed first by hardware name,
         then by logical name.
 
-        @param func : a string that uniquely characterizes the generic sensor
+        If a call to this object's is_online() method returns FALSE although
+        you are certain that the matching device is plugged, make sure that you did
+        call registerHub() at application initialization time.
+
+        @param func : a string that uniquely characterizes the generic sensor, for instance
+                RX010V01.genericSensor1.
 
         @return a YGenericSensor object allowing you to drive the generic sensor.
         """
@@ -306,7 +385,8 @@ class YGenericSensor(YSensor):
     def zeroAdjust(self):
         """
         Adjusts the signal bias so that the current signal value is need
-        precisely as zero.
+        precisely as zero. Remember to call the saveToFlash()
+        method of the module if the modification must be kept.
 
         @return YAPI.SUCCESS if the call succeeds.
 
@@ -321,6 +401,9 @@ class YGenericSensor(YSensor):
     def nextGenericSensor(self):
         """
         Continues the enumeration of generic sensors started using yFirstGenericSensor().
+        Caution: You can't make any assumption about the returned generic sensors order.
+        If you want to find a specific a generic sensor, use GenericSensor.findGenericSensor()
+        and a hardwareID or a logical name.
 
         @return a pointer to a YGenericSensor object, corresponding to
                 a generic sensor currently online, or a None pointer
@@ -335,7 +418,7 @@ class YGenericSensor(YSensor):
 
 #--- (end of YGenericSensor implementation)
 
-#--- (GenericSensor functions)
+#--- (YGenericSensor functions)
 
     @staticmethod
     def FirstGenericSensor():
@@ -369,4 +452,4 @@ class YGenericSensor(YSensor):
 
         return YGenericSensor.FindGenericSensor(serialRef.value + "." + funcIdRef.value)
 
-#--- (end of GenericSensor functions)
+#--- (end of YGenericSensor functions)
