@@ -1,41 +1,42 @@
-#*********************************************************************
-#*
-#* $Id: yocto_gps.py 23243 2016-02-23 14:13:12Z seb $
-#*
-#* Implements yFindGps(), the high-level API for Gps functions
-#*
-#* - - - - - - - - - License information: - - - - - - - - - 
-#*
-#*  Copyright (C) 2011 and beyond by Yoctopuce Sarl, Switzerland.
-#*
-#*  Yoctopuce Sarl (hereafter Licensor) grants to you a perpetual
-#*  non-exclusive license to use, modify, copy and integrate this
-#*  file into your software for the sole purpose of interfacing
-#*  with Yoctopuce products.
-#*
-#*  You may reproduce and distribute copies of this file in
-#*  source or object form, as long as the sole purpose of this
-#*  code is to interface with Yoctopuce products. You must retain
-#*  this notice in the distributed source file.
-#*
-#*  You should refer to Yoctopuce General Terms and Conditions
-#*  for additional information regarding your rights and
-#*  obligations.
-#*
-#*  THE SOFTWARE AND DOCUMENTATION ARE PROVIDED 'AS IS' WITHOUT
-#*  WARRANTY OF ANY KIND, EITHER EXPRESS OR IMPLIED, INCLUDING 
-#*  WITHOUT LIMITATION, ANY WARRANTY OF MERCHANTABILITY, FITNESS
-#*  FOR A PARTICULAR PURPOSE, TITLE AND NON-INFRINGEMENT. IN NO
-#*  EVENT SHALL LICENSOR BE LIABLE FOR ANY INCIDENTAL, SPECIAL,
-#*  INDIRECT OR CONSEQUENTIAL DAMAGES, LOST PROFITS OR LOST DATA,
-#*  COST OF PROCUREMENT OF SUBSTITUTE GOODS, TECHNOLOGY OR
-#*  SERVICES, ANY CLAIMS BY THIRD PARTIES (INCLUDING BUT NOT
-#*  LIMITED TO ANY DEFENSE THEREOF), ANY CLAIMS FOR INDEMNITY OR
-#*  CONTRIBUTION, OR OTHER SIMILAR COSTS, WHETHER ASSERTED ON THE
-#*  BASIS OF CONTRACT, TORT (INCLUDING NEGLIGENCE), BREACH OF
-#*  WARRANTY, OR OTHERWISE.
-#*
-#*********************************************************************/
+# -*- coding: utf-8 -*-
+# ********************************************************************
+#
+#  $Id: yocto_gps.py 39658 2020-03-12 15:36:29Z seb $
+#
+#  Implements yFindGps(), the high-level API for Gps functions
+#
+#  - - - - - - - - - License information: - - - - - - - - -
+#
+#  Copyright (C) 2011 and beyond by Yoctopuce Sarl, Switzerland.
+#
+#  Yoctopuce Sarl (hereafter Licensor) grants to you a perpetual
+#  non-exclusive license to use, modify, copy and integrate this
+#  file into your software for the sole purpose of interfacing
+#  with Yoctopuce products.
+#
+#  You may reproduce and distribute copies of this file in
+#  source or object form, as long as the sole purpose of this
+#  code is to interface with Yoctopuce products. You must retain
+#  this notice in the distributed source file.
+#
+#  You should refer to Yoctopuce General Terms and Conditions
+#  for additional information regarding your rights and
+#  obligations.
+#
+#  THE SOFTWARE AND DOCUMENTATION ARE PROVIDED 'AS IS' WITHOUT
+#  WARRANTY OF ANY KIND, EITHER EXPRESS OR IMPLIED, INCLUDING
+#  WITHOUT LIMITATION, ANY WARRANTY OF MERCHANTABILITY, FITNESS
+#  FOR A PARTICULAR PURPOSE, TITLE AND NON-INFRINGEMENT. IN NO
+#  EVENT SHALL LICENSOR BE LIABLE FOR ANY INCIDENTAL, SPECIAL,
+#  INDIRECT OR CONSEQUENTIAL DAMAGES, LOST PROFITS OR LOST DATA,
+#  COST OF PROCUREMENT OF SUBSTITUTE GOODS, TECHNOLOGY OR
+#  SERVICES, ANY CLAIMS BY THIRD PARTIES (INCLUDING BUT NOT
+#  LIMITED TO ANY DEFENSE THEREOF), ANY CLAIMS FOR INDEMNITY OR
+#  CONTRIBUTION, OR OTHER SIMILAR COSTS, WHETHER ASSERTED ON THE
+#  BASIS OF CONTRACT, TORT (INCLUDING NEGLIGENCE), BREACH OF
+#  WARRANTY, OR OTHERWISE.
+#
+# *********************************************************************
 
 
 __docformat__ = 'restructuredtext en'
@@ -46,20 +47,25 @@ from yoctopuce.yocto_api import *
 #noinspection PyProtectedMember
 class YGps(YFunction):
     """
-    The Gps function allows you to extract positionning
-    data from the GPS device. This class can provides
-    complete positionning information: However, if you
-    whish to define callbacks on position changes, you
+    The YGps class allows you to retrieve positioning
+    data from a GPS/GNSS sensor. This class can provides
+    complete positioning information. However, if you
+    wish to define callbacks on position changes or record
+    the position in the datalogger, you
     should use the YLatitude et YLongitude classes.
 
     """
-#--- (end of YGps class start)
+    #--- (end of YGps class start)
     #--- (YGps return codes)
     #--- (end of YGps return codes)
     #--- (YGps dlldef)
     #--- (end of YGps dlldef)
+    #--- (YGps yapiwrapper)
+    #--- (end of YGps yapiwrapper)
     #--- (YGps definitions)
     SATCOUNT_INVALID = YAPI.INVALID_LONG
+    SATPERCONST_INVALID = YAPI.INVALID_LONG
+    GPSREFRESHRATE_INVALID = YAPI.INVALID_DOUBLE
     LATITUDE_INVALID = YAPI.INVALID_STRING
     LONGITUDE_INVALID = YAPI.INVALID_STRING
     DILUTION_INVALID = YAPI.INVALID_DOUBLE
@@ -77,6 +83,14 @@ class YGps(YFunction):
     COORDSYSTEM_GPS_DM = 1
     COORDSYSTEM_GPS_D = 2
     COORDSYSTEM_INVALID = -1
+    CONSTELLATION_GNSS = 0
+    CONSTELLATION_GPS = 1
+    CONSTELLATION_GLONASS = 2
+    CONSTELLATION_GALILEO = 3
+    CONSTELLATION_GPS_GLONASS = 4
+    CONSTELLATION_GPS_GALILEO = 5
+    CONSTELLATION_GLONASS_GALILEO = 6
+    CONSTELLATION_INVALID = -1
     #--- (end of YGps definitions)
 
     def __init__(self, func):
@@ -86,7 +100,10 @@ class YGps(YFunction):
         self._callback = None
         self._isFixed = YGps.ISFIXED_INVALID
         self._satCount = YGps.SATCOUNT_INVALID
+        self._satPerConst = YGps.SATPERCONST_INVALID
+        self._gpsRefreshRate = YGps.GPSREFRESHRATE_INVALID
         self._coordSystem = YGps.COORDSYSTEM_INVALID
+        self._constellation = YGps.CONSTELLATION_INVALID
         self._latitude = YGps.LATITUDE_INVALID
         self._longitude = YGps.LONGITUDE_INVALID
         self._dilution = YGps.DILUTION_INVALID
@@ -100,47 +117,40 @@ class YGps(YFunction):
         #--- (end of YGps attributes)
 
     #--- (YGps implementation)
-    def _parseAttr(self, member):
-        if member.name == "isFixed":
-            self._isFixed = member.ivalue
-            return 1
-        if member.name == "satCount":
-            self._satCount = member.ivalue
-            return 1
-        if member.name == "coordSystem":
-            self._coordSystem = member.ivalue
-            return 1
-        if member.name == "latitude":
-            self._latitude = member.svalue
-            return 1
-        if member.name == "longitude":
-            self._longitude = member.svalue
-            return 1
-        if member.name == "dilution":
-            self._dilution = round(member.ivalue * 1000.0 / 65536.0) / 1000.0
-            return 1
-        if member.name == "altitude":
-            self._altitude = round(member.ivalue * 1000.0 / 65536.0) / 1000.0
-            return 1
-        if member.name == "groundSpeed":
-            self._groundSpeed = round(member.ivalue * 1000.0 / 65536.0) / 1000.0
-            return 1
-        if member.name == "direction":
-            self._direction = round(member.ivalue * 1000.0 / 65536.0) / 1000.0
-            return 1
-        if member.name == "unixTime":
-            self._unixTime = member.ivalue
-            return 1
-        if member.name == "dateTime":
-            self._dateTime = member.svalue
-            return 1
-        if member.name == "utcOffset":
-            self._utcOffset = member.ivalue
-            return 1
-        if member.name == "command":
-            self._command = member.svalue
-            return 1
-        super(YGps, self)._parseAttr(member)
+    def _parseAttr(self, json_val):
+        if json_val.has("isFixed"):
+            self._isFixed = (json_val.getInt("isFixed") > 0 if 1 else 0)
+        if json_val.has("satCount"):
+            self._satCount = json_val.getLong("satCount")
+        if json_val.has("satPerConst"):
+            self._satPerConst = json_val.getLong("satPerConst")
+        if json_val.has("gpsRefreshRate"):
+            self._gpsRefreshRate = round(json_val.getDouble("gpsRefreshRate") * 1000.0 / 65536.0) / 1000.0
+        if json_val.has("coordSystem"):
+            self._coordSystem = json_val.getInt("coordSystem")
+        if json_val.has("constellation"):
+            self._constellation = json_val.getInt("constellation")
+        if json_val.has("latitude"):
+            self._latitude = json_val.getString("latitude")
+        if json_val.has("longitude"):
+            self._longitude = json_val.getString("longitude")
+        if json_val.has("dilution"):
+            self._dilution = round(json_val.getDouble("dilution") * 1000.0 / 65536.0) / 1000.0
+        if json_val.has("altitude"):
+            self._altitude = round(json_val.getDouble("altitude") * 1000.0 / 65536.0) / 1000.0
+        if json_val.has("groundSpeed"):
+            self._groundSpeed = round(json_val.getDouble("groundSpeed") * 1000.0 / 65536.0) / 1000.0
+        if json_val.has("direction"):
+            self._direction = round(json_val.getDouble("direction") * 1000.0 / 65536.0) / 1000.0
+        if json_val.has("unixTime"):
+            self._unixTime = json_val.getLong("unixTime")
+        if json_val.has("dateTime"):
+            self._dateTime = json_val.getString("dateTime")
+        if json_val.has("utcOffset"):
+            self._utcOffset = json_val.getInt("utcOffset")
+        if json_val.has("command"):
+            self._command = json_val.getString("command")
+        super(YGps, self)._parseAttr(json_val)
 
     def get_isFixed(self):
         """
@@ -151,23 +161,61 @@ class YGps(YFunction):
 
         On failure, throws an exception or returns YGps.ISFIXED_INVALID.
         """
+        # res
         if self._cacheExpiration <= YAPI.GetTickCount():
-            if self.load(YAPI.DefaultCacheValidity) != YAPI.SUCCESS:
+            if self.load(YAPI._yapiContext.GetCacheValidity()) != YAPI.SUCCESS:
                 return YGps.ISFIXED_INVALID
-        return self._isFixed
+        res = self._isFixed
+        return res
 
     def get_satCount(self):
         """
-        Returns the count of visible satellites.
+        Returns the total count of satellites used to compute GPS position.
 
-        @return an integer corresponding to the count of visible satellites
+        @return an integer corresponding to the total count of satellites used to compute GPS position
 
         On failure, throws an exception or returns YGps.SATCOUNT_INVALID.
         """
+        # res
         if self._cacheExpiration <= YAPI.GetTickCount():
-            if self.load(YAPI.DefaultCacheValidity) != YAPI.SUCCESS:
+            if self.load(YAPI._yapiContext.GetCacheValidity()) != YAPI.SUCCESS:
                 return YGps.SATCOUNT_INVALID
-        return self._satCount
+        res = self._satCount
+        return res
+
+    def get_satPerConst(self):
+        """
+        Returns the count of visible satellites per constellation encoded
+        on a 32 bit integer: bits 0..5: GPS satellites count,  bits 6..11 : Glonass, bits 12..17 : Galileo.
+        this value is refreshed every 5 seconds only.
+
+        @return an integer corresponding to the count of visible satellites per constellation encoded
+                on a 32 bit integer: bits 0.
+
+        On failure, throws an exception or returns YGps.SATPERCONST_INVALID.
+        """
+        # res
+        if self._cacheExpiration <= YAPI.GetTickCount():
+            if self.load(YAPI._yapiContext.GetCacheValidity()) != YAPI.SUCCESS:
+                return YGps.SATPERCONST_INVALID
+        res = self._satPerConst
+        return res
+
+    def get_gpsRefreshRate(self):
+        """
+        Returns effective GPS data refresh frequency.
+        this value is refreshed every 5 seconds only.
+
+        @return a floating point number corresponding to effective GPS data refresh frequency
+
+        On failure, throws an exception or returns YGps.GPSREFRESHRATE_INVALID.
+        """
+        # res
+        if self._cacheExpiration <= YAPI.GetTickCount():
+            if self.load(YAPI._yapiContext.GetCacheValidity()) != YAPI.SUCCESS:
+                return YGps.GPSREFRESHRATE_INVALID
+        res = self._gpsRefreshRate
+        return res
 
     def get_coordSystem(self):
         """
@@ -178,14 +226,18 @@ class YGps(YFunction):
 
         On failure, throws an exception or returns YGps.COORDSYSTEM_INVALID.
         """
+        # res
         if self._cacheExpiration <= YAPI.GetTickCount():
-            if self.load(YAPI.DefaultCacheValidity) != YAPI.SUCCESS:
+            if self.load(YAPI._yapiContext.GetCacheValidity()) != YAPI.SUCCESS:
                 return YGps.COORDSYSTEM_INVALID
-        return self._coordSystem
+        res = self._coordSystem
+        return res
 
     def set_coordSystem(self, newval):
         """
         Changes the representation system used for positioning data.
+        Remember to call the saveToFlash() method of the module if the
+        modification must be kept.
 
         @param newval : a value among YGps.COORDSYSTEM_GPS_DMS, YGps.COORDSYSTEM_GPS_DM and
         YGps.COORDSYSTEM_GPS_D corresponding to the representation system used for positioning data
@@ -197,6 +249,44 @@ class YGps(YFunction):
         rest_val = str(newval)
         return self._setAttr("coordSystem", rest_val)
 
+    def get_constellation(self):
+        """
+        Returns the the satellites constellation used to compute
+        positioning data.
+
+        @return a value among YGps.CONSTELLATION_GNSS, YGps.CONSTELLATION_GPS, YGps.CONSTELLATION_GLONASS,
+        YGps.CONSTELLATION_GALILEO, YGps.CONSTELLATION_GPS_GLONASS, YGps.CONSTELLATION_GPS_GALILEO and
+        YGps.CONSTELLATION_GLONASS_GALILEO corresponding to the the satellites constellation used to compute
+                positioning data
+
+        On failure, throws an exception or returns YGps.CONSTELLATION_INVALID.
+        """
+        # res
+        if self._cacheExpiration <= YAPI.GetTickCount():
+            if self.load(YAPI._yapiContext.GetCacheValidity()) != YAPI.SUCCESS:
+                return YGps.CONSTELLATION_INVALID
+        res = self._constellation
+        return res
+
+    def set_constellation(self, newval):
+        """
+        Changes the satellites constellation used to compute
+        positioning data. Possible  constellations are GNSS ( = all supported constellations),
+        GPS, Glonass, Galileo , and the 3 possible pairs. This setting has  no effect on Yocto-GPS (V1).
+
+        @param newval : a value among YGps.CONSTELLATION_GNSS, YGps.CONSTELLATION_GPS,
+        YGps.CONSTELLATION_GLONASS, YGps.CONSTELLATION_GALILEO, YGps.CONSTELLATION_GPS_GLONASS,
+        YGps.CONSTELLATION_GPS_GALILEO and YGps.CONSTELLATION_GLONASS_GALILEO corresponding to the
+        satellites constellation used to compute
+                positioning data
+
+        @return YAPI.SUCCESS if the call succeeds.
+
+        On failure, throws an exception or returns a negative error code.
+        """
+        rest_val = str(newval)
+        return self._setAttr("constellation", rest_val)
+
     def get_latitude(self):
         """
         Returns the current latitude.
@@ -205,10 +295,12 @@ class YGps(YFunction):
 
         On failure, throws an exception or returns YGps.LATITUDE_INVALID.
         """
+        # res
         if self._cacheExpiration <= YAPI.GetTickCount():
-            if self.load(YAPI.DefaultCacheValidity) != YAPI.SUCCESS:
+            if self.load(YAPI._yapiContext.GetCacheValidity()) != YAPI.SUCCESS:
                 return YGps.LATITUDE_INVALID
-        return self._latitude
+        res = self._latitude
+        return res
 
     def get_longitude(self):
         """
@@ -218,10 +310,12 @@ class YGps(YFunction):
 
         On failure, throws an exception or returns YGps.LONGITUDE_INVALID.
         """
+        # res
         if self._cacheExpiration <= YAPI.GetTickCount():
-            if self.load(YAPI.DefaultCacheValidity) != YAPI.SUCCESS:
+            if self.load(YAPI._yapiContext.GetCacheValidity()) != YAPI.SUCCESS:
                 return YGps.LONGITUDE_INVALID
-        return self._longitude
+        res = self._longitude
+        return res
 
     def get_dilution(self):
         """
@@ -233,10 +327,12 @@ class YGps(YFunction):
 
         On failure, throws an exception or returns YGps.DILUTION_INVALID.
         """
+        # res
         if self._cacheExpiration <= YAPI.GetTickCount():
-            if self.load(YAPI.DefaultCacheValidity) != YAPI.SUCCESS:
+            if self.load(YAPI._yapiContext.GetCacheValidity()) != YAPI.SUCCESS:
                 return YGps.DILUTION_INVALID
-        return self._dilution
+        res = self._dilution
+        return res
 
     def get_altitude(self):
         """
@@ -247,10 +343,12 @@ class YGps(YFunction):
 
         On failure, throws an exception or returns YGps.ALTITUDE_INVALID.
         """
+        # res
         if self._cacheExpiration <= YAPI.GetTickCount():
-            if self.load(YAPI.DefaultCacheValidity) != YAPI.SUCCESS:
+            if self.load(YAPI._yapiContext.GetCacheValidity()) != YAPI.SUCCESS:
                 return YGps.ALTITUDE_INVALID
-        return self._altitude
+        res = self._altitude
+        return res
 
     def get_groundSpeed(self):
         """
@@ -260,10 +358,12 @@ class YGps(YFunction):
 
         On failure, throws an exception or returns YGps.GROUNDSPEED_INVALID.
         """
+        # res
         if self._cacheExpiration <= YAPI.GetTickCount():
-            if self.load(YAPI.DefaultCacheValidity) != YAPI.SUCCESS:
+            if self.load(YAPI._yapiContext.GetCacheValidity()) != YAPI.SUCCESS:
                 return YGps.GROUNDSPEED_INVALID
-        return self._groundSpeed
+        res = self._groundSpeed
+        return res
 
     def get_direction(self):
         """
@@ -275,10 +375,12 @@ class YGps(YFunction):
 
         On failure, throws an exception or returns YGps.DIRECTION_INVALID.
         """
+        # res
         if self._cacheExpiration <= YAPI.GetTickCount():
-            if self.load(YAPI.DefaultCacheValidity) != YAPI.SUCCESS:
+            if self.load(YAPI._yapiContext.GetCacheValidity()) != YAPI.SUCCESS:
                 return YGps.DIRECTION_INVALID
-        return self._direction
+        res = self._direction
+        return res
 
     def get_unixTime(self):
         """
@@ -290,10 +392,12 @@ class YGps(YFunction):
 
         On failure, throws an exception or returns YGps.UNIXTIME_INVALID.
         """
+        # res
         if self._cacheExpiration <= YAPI.GetTickCount():
-            if self.load(YAPI.DefaultCacheValidity) != YAPI.SUCCESS:
+            if self.load(YAPI._yapiContext.GetCacheValidity()) != YAPI.SUCCESS:
                 return YGps.UNIXTIME_INVALID
-        return self._unixTime
+        res = self._unixTime
+        return res
 
     def get_dateTime(self):
         """
@@ -303,10 +407,12 @@ class YGps(YFunction):
 
         On failure, throws an exception or returns YGps.DATETIME_INVALID.
         """
+        # res
         if self._cacheExpiration <= YAPI.GetTickCount():
-            if self.load(YAPI.DefaultCacheValidity) != YAPI.SUCCESS:
+            if self.load(YAPI._yapiContext.GetCacheValidity()) != YAPI.SUCCESS:
                 return YGps.DATETIME_INVALID
-        return self._dateTime
+        res = self._dateTime
+        return res
 
     def get_utcOffset(self):
         """
@@ -316,16 +422,20 @@ class YGps(YFunction):
 
         On failure, throws an exception or returns YGps.UTCOFFSET_INVALID.
         """
+        # res
         if self._cacheExpiration <= YAPI.GetTickCount():
-            if self.load(YAPI.DefaultCacheValidity) != YAPI.SUCCESS:
+            if self.load(YAPI._yapiContext.GetCacheValidity()) != YAPI.SUCCESS:
                 return YGps.UTCOFFSET_INVALID
-        return self._utcOffset
+        res = self._utcOffset
+        return res
 
     def set_utcOffset(self, newval):
         """
         Changes the number of seconds between current time and UTC time (time zone).
         The timezone is automatically rounded to the nearest multiple of 15 minutes.
         If current UTC time is known, the current time is automatically be updated according to the selected time zone.
+        Remember to call the saveToFlash() method of the module if the
+        modification must be kept.
 
         @param newval : an integer corresponding to the number of seconds between current time and UTC time (time zone)
 
@@ -337,10 +447,12 @@ class YGps(YFunction):
         return self._setAttr("utcOffset", rest_val)
 
     def get_command(self):
+        # res
         if self._cacheExpiration <= YAPI.GetTickCount():
-            if self.load(YAPI.DefaultCacheValidity) != YAPI.SUCCESS:
+            if self.load(YAPI._yapiContext.GetCacheValidity()) != YAPI.SUCCESS:
                 return YGps.COMMAND_INVALID
-        return self._command
+        res = self._command
+        return res
 
     def set_command(self, newval):
         rest_val = newval
@@ -349,7 +461,7 @@ class YGps(YFunction):
     @staticmethod
     def FindGps(func):
         """
-        Retrieves a GPS for a given identifier.
+        Retrieves a geolocalization module for a given identifier.
         The identifier can be specified using several formats:
         <ul>
         <li>FunctionLogicalName</li>
@@ -359,17 +471,22 @@ class YGps(YFunction):
         <li>ModuleLogicalName.FunctionLogicalName</li>
         </ul>
 
-        This function does not require that the GPS is online at the time
+        This function does not require that the geolocalization module is online at the time
         it is invoked. The returned object is nevertheless valid.
-        Use the method YGps.isOnline() to test if the GPS is
+        Use the method YGps.isOnline() to test if the geolocalization module is
         indeed online at a given time. In case of ambiguity when looking for
-        a GPS by logical name, no error is notified: the first instance
+        a geolocalization module by logical name, no error is notified: the first instance
         found is returned. The search is performed first by hardware name,
         then by logical name.
 
-        @param func : a string that uniquely characterizes the GPS
+        If a call to this object's is_online() method returns FALSE although
+        you are certain that the matching device is plugged, make sure that you did
+        call registerHub() at application initialization time.
 
-        @return a YGps object allowing you to drive the GPS.
+        @param func : a string that uniquely characterizes the geolocalization module, for instance
+                YGNSSMK2.gps.
+
+        @return a YGps object allowing you to drive the geolocalization module.
         """
         # obj
         obj = YFunction._FindFromCache("Gps", func)
@@ -380,11 +497,14 @@ class YGps(YFunction):
 
     def nextGps(self):
         """
-        Continues the enumeration of GPS started using yFirstGps().
+        Continues the enumeration of geolocalization modules started using yFirstGps().
+        Caution: You can't make any assumption about the returned geolocalization modules order.
+        If you want to find a specific a geolocalization module, use Gps.findGps()
+        and a hardwareID or a logical name.
 
         @return a pointer to a YGps object, corresponding to
-                a GPS currently online, or a None pointer
-                if there are no more GPS to enumerate.
+                a geolocalization module currently online, or a None pointer
+                if there are no more geolocalization modules to enumerate.
         """
         hwidRef = YRefParam()
         if YAPI.YISERR(self._nextFunction(hwidRef)):
@@ -395,17 +515,17 @@ class YGps(YFunction):
 
 #--- (end of YGps implementation)
 
-#--- (Gps functions)
+#--- (YGps functions)
 
     @staticmethod
     def FirstGps():
         """
-        Starts the enumeration of GPS currently accessible.
+        Starts the enumeration of geolocalization modules currently accessible.
         Use the method YGps.nextGps() to iterate on
-        next GPS.
+        next geolocalization modules.
 
         @return a pointer to a YGps object, corresponding to
-                the first GPS currently online, or a None pointer
+                the first geolocalization module currently online, or a None pointer
                 if there are none.
         """
         devRef = YRefParam()
@@ -429,4 +549,4 @@ class YGps(YFunction):
 
         return YGps.FindGps(serialRef.value + "." + funcIdRef.value)
 
-#--- (end of Gps functions)
+#--- (end of YGps functions)
