@@ -1,7 +1,7 @@
 # -*- coding: utf-8 -*-
 #*********************************************************************
 #*
-#* $Id: yocto_gyro.py 38899 2019-12-20 17:21:03Z mvuilleu $
+#* $Id: yocto_gyro.py 42951 2020-12-14 09:43:29Z seb $
 #*
 #* Implements yFindGyro(), the high-level API for Gyro functions
 #*
@@ -237,9 +237,9 @@ class YGyro(YSensor):
 
     def get_bandwidth(self):
         """
-        Returns the measure update frequency, measured in Hz (Yocto-3D-V2 only).
+        Returns the measure update frequency, measured in Hz.
 
-        @return an integer corresponding to the measure update frequency, measured in Hz (Yocto-3D-V2 only)
+        @return an integer corresponding to the measure update frequency, measured in Hz
 
         On failure, throws an exception or returns YGyro.BANDWIDTH_INVALID.
         """
@@ -252,12 +252,12 @@ class YGyro(YSensor):
 
     def set_bandwidth(self, newval):
         """
-        Changes the measure update frequency, measured in Hz (Yocto-3D-V2 only). When the
+        Changes the measure update frequency, measured in Hz. When the
         frequency is lower, the device performs averaging.
         Remember to call the saveToFlash()
         method of the module if the modification must be kept.
 
-        @param newval : an integer corresponding to the measure update frequency, measured in Hz (Yocto-3D-V2 only)
+        @param newval : an integer corresponding to the measure update frequency, measured in Hz
 
         @return YAPI.SUCCESS if the call succeeds.
 
