@@ -1,41 +1,42 @@
-#*********************************************************************
-#*
-#* $Id: yocto_motor.py 23243 2016-02-23 14:13:12Z seb $
-#*
-#* Implements yFindMotor(), the high-level API for Motor functions
-#*
-#* - - - - - - - - - License information: - - - - - - - - - 
-#*
-#*  Copyright (C) 2011 and beyond by Yoctopuce Sarl, Switzerland.
-#*
-#*  Yoctopuce Sarl (hereafter Licensor) grants to you a perpetual
-#*  non-exclusive license to use, modify, copy and integrate this
-#*  file into your software for the sole purpose of interfacing
-#*  with Yoctopuce products.
-#*
-#*  You may reproduce and distribute copies of this file in
-#*  source or object form, as long as the sole purpose of this
-#*  code is to interface with Yoctopuce products. You must retain
-#*  this notice in the distributed source file.
-#*
-#*  You should refer to Yoctopuce General Terms and Conditions
-#*  for additional information regarding your rights and
-#*  obligations.
-#*
-#*  THE SOFTWARE AND DOCUMENTATION ARE PROVIDED 'AS IS' WITHOUT
-#*  WARRANTY OF ANY KIND, EITHER EXPRESS OR IMPLIED, INCLUDING 
-#*  WITHOUT LIMITATION, ANY WARRANTY OF MERCHANTABILITY, FITNESS
-#*  FOR A PARTICULAR PURPOSE, TITLE AND NON-INFRINGEMENT. IN NO
-#*  EVENT SHALL LICENSOR BE LIABLE FOR ANY INCIDENTAL, SPECIAL,
-#*  INDIRECT OR CONSEQUENTIAL DAMAGES, LOST PROFITS OR LOST DATA,
-#*  COST OF PROCUREMENT OF SUBSTITUTE GOODS, TECHNOLOGY OR
-#*  SERVICES, ANY CLAIMS BY THIRD PARTIES (INCLUDING BUT NOT
-#*  LIMITED TO ANY DEFENSE THEREOF), ANY CLAIMS FOR INDEMNITY OR
-#*  CONTRIBUTION, OR OTHER SIMILAR COSTS, WHETHER ASSERTED ON THE
-#*  BASIS OF CONTRACT, TORT (INCLUDING NEGLIGENCE), BREACH OF
-#*  WARRANTY, OR OTHERWISE.
-#*
-#*********************************************************************/
+# -*- coding: utf-8 -*-
+# ********************************************************************
+#
+#  $Id: yocto_motor.py 38899 2019-12-20 17:21:03Z mvuilleu $
+#
+#  Implements yFindMotor(), the high-level API for Motor functions
+#
+#  - - - - - - - - - License information: - - - - - - - - -
+#
+#  Copyright (C) 2011 and beyond by Yoctopuce Sarl, Switzerland.
+#
+#  Yoctopuce Sarl (hereafter Licensor) grants to you a perpetual
+#  non-exclusive license to use, modify, copy and integrate this
+#  file into your software for the sole purpose of interfacing
+#  with Yoctopuce products.
+#
+#  You may reproduce and distribute copies of this file in
+#  source or object form, as long as the sole purpose of this
+#  code is to interface with Yoctopuce products. You must retain
+#  this notice in the distributed source file.
+#
+#  You should refer to Yoctopuce General Terms and Conditions
+#  for additional information regarding your rights and
+#  obligations.
+#
+#  THE SOFTWARE AND DOCUMENTATION ARE PROVIDED 'AS IS' WITHOUT
+#  WARRANTY OF ANY KIND, EITHER EXPRESS OR IMPLIED, INCLUDING
+#  WITHOUT LIMITATION, ANY WARRANTY OF MERCHANTABILITY, FITNESS
+#  FOR A PARTICULAR PURPOSE, TITLE AND NON-INFRINGEMENT. IN NO
+#  EVENT SHALL LICENSOR BE LIABLE FOR ANY INCIDENTAL, SPECIAL,
+#  INDIRECT OR CONSEQUENTIAL DAMAGES, LOST PROFITS OR LOST DATA,
+#  COST OF PROCUREMENT OF SUBSTITUTE GOODS, TECHNOLOGY OR
+#  SERVICES, ANY CLAIMS BY THIRD PARTIES (INCLUDING BUT NOT
+#  LIMITED TO ANY DEFENSE THEREOF), ANY CLAIMS FOR INDEMNITY OR
+#  CONTRIBUTION, OR OTHER SIMILAR COSTS, WHETHER ASSERTED ON THE
+#  BASIS OF CONTRACT, TORT (INCLUDING NEGLIGENCE), BREACH OF
+#  WARRANTY, OR OTHERWISE.
+#
+# *********************************************************************
 
 
 __docformat__ = 'restructuredtext en'
@@ -46,25 +47,27 @@ from yoctopuce.yocto_api import *
 #noinspection PyProtectedMember
 class YMotor(YFunction):
     """
-    Yoctopuce application programming interface allows you to drive the
+    The YMotor class allows you to drive a DC motor. It can be used to configure the
     power sent to the motor to make it turn both ways, but also to drive accelerations
     and decelerations. The motor will then accelerate automatically: you will not
     have to monitor it. The API also allows to slow down the motor by shortening
     its terminals: the motor will then act as an electromagnetic brake.
 
     """
-#--- (end of YMotor class start)
+    #--- (end of YMotor class start)
     #--- (YMotor return codes)
     #--- (end of YMotor return codes)
     #--- (YMotor dlldef)
     #--- (end of YMotor dlldef)
+    #--- (YMotor yapiwrapper)
+    #--- (end of YMotor yapiwrapper)
     #--- (YMotor definitions)
     DRIVINGFORCE_INVALID = YAPI.INVALID_DOUBLE
     BRAKINGFORCE_INVALID = YAPI.INVALID_DOUBLE
     CUTOFFVOLTAGE_INVALID = YAPI.INVALID_DOUBLE
-    OVERCURRENTLIMIT_INVALID = YAPI.INVALID_INT
+    OVERCURRENTLIMIT_INVALID = YAPI.INVALID_UINT
     FREQUENCY_INVALID = YAPI.INVALID_DOUBLE
-    STARTERTIME_INVALID = YAPI.INVALID_INT
+    STARTERTIME_INVALID = YAPI.INVALID_UINT
     FAILSAFETIMEOUT_INVALID = YAPI.INVALID_UINT
     COMMAND_INVALID = YAPI.INVALID_STRING
     MOTORSTATUS_IDLE = 0
@@ -95,35 +98,26 @@ class YMotor(YFunction):
         #--- (end of YMotor attributes)
 
     #--- (YMotor implementation)
-    def _parseAttr(self, member):
-        if member.name == "motorStatus":
-            self._motorStatus = member.ivalue
-            return 1
-        if member.name == "drivingForce":
-            self._drivingForce = round(member.ivalue * 1000.0 / 65536.0) / 1000.0
-            return 1
-        if member.name == "brakingForce":
-            self._brakingForce = round(member.ivalue * 1000.0 / 65536.0) / 1000.0
-            return 1
-        if member.name == "cutOffVoltage":
-            self._cutOffVoltage = round(member.ivalue * 1000.0 / 65536.0) / 1000.0
-            return 1
-        if member.name == "overCurrentLimit":
-            self._overCurrentLimit = member.ivalue
-            return 1
-        if member.name == "frequency":
-            self._frequency = round(member.ivalue * 1000.0 / 65536.0) / 1000.0
-            return 1
-        if member.name == "starterTime":
-            self._starterTime = member.ivalue
-            return 1
-        if member.name == "failSafeTimeout":
-            self._failSafeTimeout = member.ivalue
-            return 1
-        if member.name == "command":
-            self._command = member.svalue
-            return 1
-        super(YMotor, self)._parseAttr(member)
+    def _parseAttr(self, json_val):
+        if json_val.has("motorStatus"):
+            self._motorStatus = json_val.getInt("motorStatus")
+        if json_val.has("drivingForce"):
+            self._drivingForce = round(json_val.getDouble("drivingForce") * 1000.0 / 65536.0) / 1000.0
+        if json_val.has("brakingForce"):
+            self._brakingForce = round(json_val.getDouble("brakingForce") * 1000.0 / 65536.0) / 1000.0
+        if json_val.has("cutOffVoltage"):
+            self._cutOffVoltage = round(json_val.getDouble("cutOffVoltage") * 1000.0 / 65536.0) / 1000.0
+        if json_val.has("overCurrentLimit"):
+            self._overCurrentLimit = json_val.getInt("overCurrentLimit")
+        if json_val.has("frequency"):
+            self._frequency = round(json_val.getDouble("frequency") * 1000.0 / 65536.0) / 1000.0
+        if json_val.has("starterTime"):
+            self._starterTime = json_val.getInt("starterTime")
+        if json_val.has("failSafeTimeout"):
+            self._failSafeTimeout = json_val.getInt("failSafeTimeout")
+        if json_val.has("command"):
+            self._command = json_val.getString("command")
+        super(YMotor, self)._parseAttr(json_val)
 
     def get_motorStatus(self):
         """
@@ -133,7 +127,7 @@ class YMotor(YFunction):
         BACKWD when the controller is driving the motor backward;
         BRAKE  when the controller is braking;
         LOVOLT when the controller has detected a low voltage condition;
-        HICURR when the controller has detected an overcurrent condition;
+        HICURR when the controller has detected an over current condition;
         HIHEAT when the controller has detected an overheat condition;
         FAILSF when the controller switched on the failsafe security.
 
@@ -146,10 +140,12 @@ class YMotor(YFunction):
 
         On failure, throws an exception or returns YMotor.MOTORSTATUS_INVALID.
         """
+        # res
         if self._cacheExpiration <= YAPI.GetTickCount():
-            if self.load(YAPI.DefaultCacheValidity) != YAPI.SUCCESS:
+            if self.load(YAPI._yapiContext.GetCacheValidity()) != YAPI.SUCCESS:
                 return YMotor.MOTORSTATUS_INVALID
-        return self._motorStatus
+        res = self._motorStatus
+        return res
 
     def set_motorStatus(self, newval):
         rest_val = str(newval)
@@ -181,10 +177,12 @@ class YMotor(YFunction):
 
         On failure, throws an exception or returns YMotor.DRIVINGFORCE_INVALID.
         """
+        # res
         if self._cacheExpiration <= YAPI.GetTickCount():
-            if self.load(YAPI.DefaultCacheValidity) != YAPI.SUCCESS:
+            if self.load(YAPI._yapiContext.GetCacheValidity()) != YAPI.SUCCESS:
                 return YMotor.DRIVINGFORCE_INVALID
-        return self._drivingForce
+        res = self._drivingForce
+        return res
 
     def set_brakingForce(self, newval):
         """
@@ -211,10 +209,12 @@ class YMotor(YFunction):
 
         On failure, throws an exception or returns YMotor.BRAKINGFORCE_INVALID.
         """
+        # res
         if self._cacheExpiration <= YAPI.GetTickCount():
-            if self.load(YAPI.DefaultCacheValidity) != YAPI.SUCCESS:
+            if self.load(YAPI._yapiContext.GetCacheValidity()) != YAPI.SUCCESS:
                 return YMotor.BRAKINGFORCE_INVALID
-        return self._brakingForce
+        res = self._brakingForce
+        return res
 
     def set_cutOffVoltage(self, newval):
         """
@@ -223,6 +223,8 @@ class YMotor(YFunction):
         occur when drawing current from an "empty" battery.
         Note that whatever the cutoff threshold, the controller switches to undervoltage
         error state if the power supply goes under 3V, even for a very brief time.
+        Remember to call the saveToFlash()
+        method of the module if the modification must be kept.
 
         @param newval : a floating point number corresponding to the threshold voltage under which the
         controller automatically switches to error state
@@ -247,10 +249,12 @@ class YMotor(YFunction):
 
         On failure, throws an exception or returns YMotor.CUTOFFVOLTAGE_INVALID.
         """
+        # res
         if self._cacheExpiration <= YAPI.GetTickCount():
-            if self.load(YAPI.DefaultCacheValidity) != YAPI.SUCCESS:
+            if self.load(YAPI._yapiContext.GetCacheValidity()) != YAPI.SUCCESS:
                 return YMotor.CUTOFFVOLTAGE_INVALID
-        return self._cutOffVoltage
+        res = self._cutOffVoltage
+        return res
 
     def get_overCurrentLimit(self):
         """
@@ -262,17 +266,20 @@ class YMotor(YFunction):
 
         On failure, throws an exception or returns YMotor.OVERCURRENTLIMIT_INVALID.
         """
+        # res
         if self._cacheExpiration <= YAPI.GetTickCount():
-            if self.load(YAPI.DefaultCacheValidity) != YAPI.SUCCESS:
+            if self.load(YAPI._yapiContext.GetCacheValidity()) != YAPI.SUCCESS:
                 return YMotor.OVERCURRENTLIMIT_INVALID
-        return self._overCurrentLimit
+        res = self._overCurrentLimit
+        return res
 
     def set_overCurrentLimit(self, newval):
         """
         Changes the current threshold (in mA) above which the controller automatically
         switches to error state. A zero value means that there is no limit. Note that whatever the
         current limit is, the controller switches to OVERCURRENT status if the current
-        goes above 32A, even for a very brief time.
+        goes above 32A, even for a very brief time. Remember to call the saveToFlash()
+        method of the module if the modification must be kept.
 
         @param newval : an integer corresponding to the current threshold (in mA) above which the
         controller automatically
@@ -290,7 +297,8 @@ class YMotor(YFunction):
         Changes the PWM frequency used to control the motor. Low frequency is usually
         more efficient and may help the motor to start, but an audible noise might be
         generated. A higher frequency reduces the noise, but more energy is converted
-        into heat.
+        into heat. Remember to call the saveToFlash()
+        method of the module if the modification must be kept.
 
         @param newval : a floating point number corresponding to the PWM frequency used to control the motor
 
@@ -309,10 +317,12 @@ class YMotor(YFunction):
 
         On failure, throws an exception or returns YMotor.FREQUENCY_INVALID.
         """
+        # res
         if self._cacheExpiration <= YAPI.GetTickCount():
-            if self.load(YAPI.DefaultCacheValidity) != YAPI.SUCCESS:
+            if self.load(YAPI._yapiContext.GetCacheValidity()) != YAPI.SUCCESS:
                 return YMotor.FREQUENCY_INVALID
-        return self._frequency
+        res = self._frequency
+        return res
 
     def get_starterTime(self):
         """
@@ -325,15 +335,18 @@ class YMotor(YFunction):
 
         On failure, throws an exception or returns YMotor.STARTERTIME_INVALID.
         """
+        # res
         if self._cacheExpiration <= YAPI.GetTickCount():
-            if self.load(YAPI.DefaultCacheValidity) != YAPI.SUCCESS:
+            if self.load(YAPI._yapiContext.GetCacheValidity()) != YAPI.SUCCESS:
                 return YMotor.STARTERTIME_INVALID
-        return self._starterTime
+        res = self._starterTime
+        return res
 
     def set_starterTime(self, newval):
         """
         Changes the duration (in ms) during which the motor is driven at low frequency to help
-        it start up.
+        it start up. Remember to call the saveToFlash()
+        method of the module if the modification must be kept.
 
         @param newval : an integer corresponding to the duration (in ms) during which the motor is driven
         at low frequency to help
@@ -359,10 +372,12 @@ class YMotor(YFunction):
 
         On failure, throws an exception or returns YMotor.FAILSAFETIMEOUT_INVALID.
         """
+        # res
         if self._cacheExpiration <= YAPI.GetTickCount():
-            if self.load(YAPI.DefaultCacheValidity) != YAPI.SUCCESS:
+            if self.load(YAPI._yapiContext.GetCacheValidity()) != YAPI.SUCCESS:
                 return YMotor.FAILSAFETIMEOUT_INVALID
-        return self._failSafeTimeout
+        res = self._failSafeTimeout
+        return res
 
     def set_failSafeTimeout(self, newval):
         """
@@ -370,6 +385,8 @@ class YMotor(YFunction):
         receiving any instruction from the control process. When this delay has elapsed,
         the controller automatically stops the motor and switches to FAILSAFE error.
         Failsafe security is disabled when the value is zero.
+        Remember to call the saveToFlash()
+        method of the module if the modification must be kept.
 
         @param newval : an integer corresponding to the delay in milliseconds allowed for the controller to
         run autonomously without
@@ -383,10 +400,12 @@ class YMotor(YFunction):
         return self._setAttr("failSafeTimeout", rest_val)
 
     def get_command(self):
+        # res
         if self._cacheExpiration <= YAPI.GetTickCount():
-            if self.load(YAPI.DefaultCacheValidity) != YAPI.SUCCESS:
+            if self.load(YAPI._yapiContext.GetCacheValidity()) != YAPI.SUCCESS:
                 return YMotor.COMMAND_INVALID
-        return self._command
+        res = self._command
+        return res
 
     def set_command(self, newval):
         rest_val = newval
@@ -413,7 +432,12 @@ class YMotor(YFunction):
         found is returned. The search is performed first by hardware name,
         then by logical name.
 
-        @param func : a string that uniquely characterizes the motor
+        If a call to this object's is_online() method returns FALSE although
+        you are certain that the matching device is plugged, make sure that you did
+        call registerHub() at application initialization time.
+
+        @param func : a string that uniquely characterizes the motor, for instance
+                MOTORCTL.motor.
 
         @return a YMotor object allowing you to drive the motor.
         """
@@ -429,22 +453,20 @@ class YMotor(YFunction):
         Rearms the controller failsafe timer. When the motor is running and the failsafe feature
         is active, this function should be called periodically to prove that the control process
         is running properly. Otherwise, the motor is automatically stopped after the specified
-        timeout. Calling a motor <i>set</i> function implicitely rearms the failsafe timer.
+        timeout. Calling a motor <i>set</i> function implicitly rearms the failsafe timer.
         """
-        # // may throw an exception
         return self.set_command("K")
 
     def resetStatus(self):
         """
-        Reset the controller state to IDLE. This function must be invoked explicitely
+        Reset the controller state to IDLE. This function must be invoked explicitly
         after any error condition is signaled.
         """
-        # // may throw an exception
         return self.set_motorStatus(YMotor.MOTORSTATUS_IDLE)
 
     def drivingForceMove(self, targetPower, delay):
         """
-        Changes progressively the power sent to the moteur for a specific duration.
+        Changes progressively the power sent to the motor for a specific duration.
 
         @param targetPower : desired motor power, in percents (between -100% and +100%)
         @param delay : duration (in ms) of the transition
@@ -471,6 +493,9 @@ class YMotor(YFunction):
     def nextMotor(self):
         """
         Continues the enumeration of motors started using yFirstMotor().
+        Caution: You can't make any assumption about the returned motors order.
+        If you want to find a specific a motor, use Motor.findMotor()
+        and a hardwareID or a logical name.
 
         @return a pointer to a YMotor object, corresponding to
                 a motor currently online, or a None pointer
@@ -485,7 +510,7 @@ class YMotor(YFunction):
 
 #--- (end of YMotor implementation)
 
-#--- (Motor functions)
+#--- (YMotor functions)
 
     @staticmethod
     def FirstMotor():
@@ -519,4 +544,4 @@ class YMotor(YFunction):
 
         return YMotor.FindMotor(serialRef.value + "." + funcIdRef.value)
 
-#--- (end of Motor functions)
+#--- (end of YMotor functions)
