@@ -1,41 +1,42 @@
-#*********************************************************************
-#*
-#* $Id: yocto_relay.py 23243 2016-02-23 14:13:12Z seb $
-#*
-#* Implements yFindRelay(), the high-level API for Relay functions
-#*
-#* - - - - - - - - - License information: - - - - - - - - - 
-#*
-#*  Copyright (C) 2011 and beyond by Yoctopuce Sarl, Switzerland.
-#*
-#*  Yoctopuce Sarl (hereafter Licensor) grants to you a perpetual
-#*  non-exclusive license to use, modify, copy and integrate this
-#*  file into your software for the sole purpose of interfacing
-#*  with Yoctopuce products.
-#*
-#*  You may reproduce and distribute copies of this file in
-#*  source or object form, as long as the sole purpose of this
-#*  code is to interface with Yoctopuce products. You must retain
-#*  this notice in the distributed source file.
-#*
-#*  You should refer to Yoctopuce General Terms and Conditions
-#*  for additional information regarding your rights and
-#*  obligations.
-#*
-#*  THE SOFTWARE AND DOCUMENTATION ARE PROVIDED 'AS IS' WITHOUT
-#*  WARRANTY OF ANY KIND, EITHER EXPRESS OR IMPLIED, INCLUDING 
-#*  WITHOUT LIMITATION, ANY WARRANTY OF MERCHANTABILITY, FITNESS
-#*  FOR A PARTICULAR PURPOSE, TITLE AND NON-INFRINGEMENT. IN NO
-#*  EVENT SHALL LICENSOR BE LIABLE FOR ANY INCIDENTAL, SPECIAL,
-#*  INDIRECT OR CONSEQUENTIAL DAMAGES, LOST PROFITS OR LOST DATA,
-#*  COST OF PROCUREMENT OF SUBSTITUTE GOODS, TECHNOLOGY OR
-#*  SERVICES, ANY CLAIMS BY THIRD PARTIES (INCLUDING BUT NOT
-#*  LIMITED TO ANY DEFENSE THEREOF), ANY CLAIMS FOR INDEMNITY OR
-#*  CONTRIBUTION, OR OTHER SIMILAR COSTS, WHETHER ASSERTED ON THE
-#*  BASIS OF CONTRACT, TORT (INCLUDING NEGLIGENCE), BREACH OF
-#*  WARRANTY, OR OTHERWISE.
-#*
-#*********************************************************************/
+# -*- coding: utf-8 -*-
+# ********************************************************************
+#
+#  $Id: yocto_relay.py 41109 2020-06-29 12:40:42Z seb $
+#
+#  Implements yFindRelay(), the high-level API for Relay functions
+#
+#  - - - - - - - - - License information: - - - - - - - - -
+#
+#  Copyright (C) 2011 and beyond by Yoctopuce Sarl, Switzerland.
+#
+#  Yoctopuce Sarl (hereafter Licensor) grants to you a perpetual
+#  non-exclusive license to use, modify, copy and integrate this
+#  file into your software for the sole purpose of interfacing
+#  with Yoctopuce products.
+#
+#  You may reproduce and distribute copies of this file in
+#  source or object form, as long as the sole purpose of this
+#  code is to interface with Yoctopuce products. You must retain
+#  this notice in the distributed source file.
+#
+#  You should refer to Yoctopuce General Terms and Conditions
+#  for additional information regarding your rights and
+#  obligations.
+#
+#  THE SOFTWARE AND DOCUMENTATION ARE PROVIDED 'AS IS' WITHOUT
+#  WARRANTY OF ANY KIND, EITHER EXPRESS OR IMPLIED, INCLUDING
+#  WITHOUT LIMITATION, ANY WARRANTY OF MERCHANTABILITY, FITNESS
+#  FOR A PARTICULAR PURPOSE, TITLE AND NON-INFRINGEMENT. IN NO
+#  EVENT SHALL LICENSOR BE LIABLE FOR ANY INCIDENTAL, SPECIAL,
+#  INDIRECT OR CONSEQUENTIAL DAMAGES, LOST PROFITS OR LOST DATA,
+#  COST OF PROCUREMENT OF SUBSTITUTE GOODS, TECHNOLOGY OR
+#  SERVICES, ANY CLAIMS BY THIRD PARTIES (INCLUDING BUT NOT
+#  LIMITED TO ANY DEFENSE THEREOF), ANY CLAIMS FOR INDEMNITY OR
+#  CONTRIBUTION, OR OTHER SIMILAR COSTS, WHETHER ASSERTED ON THE
+#  BASIS OF CONTRACT, TORT (INCLUDING NEGLIGENCE), BREACH OF
+#  WARRANTY, OR OTHERWISE.
+#
+# *********************************************************************
 
 
 __docformat__ = 'restructuredtext en'
@@ -46,20 +47,21 @@ from yoctopuce.yocto_api import *
 #noinspection PyProtectedMember
 class YRelay(YFunction):
     """
-    The Yoctopuce application programming interface allows you to switch the relay state.
-    This change is not persistent: the relay will automatically return to its idle position
-    whenever power is lost or if the module is restarted.
-    The library can also generate automatically short pulses of determined duration.
+    The YRelay class allows you to drive a Yoctopuce relay or optocoupled output.
+    It can be used to simply switch the output on or off, but also to automatically generate short
+    pulses of determined duration.
     On devices with two output for each relay (double throw), the two outputs are named A and B,
-    with output A corresponding to the idle position (at power off) and the output B corresponding to the
-    active state. If you prefer the alternate default state, simply switch your cables on the board.
+    with output A corresponding to the idle position (normally closed) and the output B corresponding to the
+    active state (normally open).
 
     """
-#--- (end of YRelay class start)
+    #--- (end of YRelay class start)
     #--- (YRelay return codes)
     #--- (end of YRelay return codes)
     #--- (YRelay dlldef)
     #--- (end of YRelay dlldef)
+    #--- (YRelay yapiwrapper)
+    #--- (end of YRelay yapiwrapper)
     #--- (YRelay definitions)
     MAXTIMEONSTATEA_INVALID = YAPI.INVALID_LONG
     MAXTIMEONSTATEB_INVALID = YAPI.INVALID_LONG
@@ -91,44 +93,35 @@ class YRelay(YFunction):
         self._pulseTimer = YRelay.PULSETIMER_INVALID
         self._delayedPulseTimer = YRelay.DELAYEDPULSETIMER_INVALID
         self._countdown = YRelay.COUNTDOWN_INVALID
+        self._firm = 0
         #--- (end of YRelay attributes)
 
     #--- (YRelay implementation)
-    def _parseAttr(self, member):
-        if member.name == "state":
-            self._state = member.ivalue
-            return 1
-        if member.name == "stateAtPowerOn":
-            self._stateAtPowerOn = member.ivalue
-            return 1
-        if member.name == "maxTimeOnStateA":
-            self._maxTimeOnStateA = member.ivalue
-            return 1
-        if member.name == "maxTimeOnStateB":
-            self._maxTimeOnStateB = member.ivalue
-            return 1
-        if member.name == "output":
-            self._output = member.ivalue
-            return 1
-        if member.name == "pulseTimer":
-            self._pulseTimer = member.ivalue
-            return 1
-        if member.name == "delayedPulseTimer":
-            if member.recordtype != YAPI.TJSONRECORDTYPE.JSON_STRUCT:
-                self._delayedPulseTimer = -1
+    def _parseAttr(self, json_val):
+        if json_val.has("state"):
+            self._state = (json_val.getInt("state") > 0 if 1 else 0)
+        if json_val.has("stateAtPowerOn"):
+            self._stateAtPowerOn = json_val.getInt("stateAtPowerOn")
+        if json_val.has("maxTimeOnStateA"):
+            self._maxTimeOnStateA = json_val.getLong("maxTimeOnStateA")
+        if json_val.has("maxTimeOnStateB"):
+            self._maxTimeOnStateB = json_val.getLong("maxTimeOnStateB")
+        if json_val.has("output"):
+            self._output = (json_val.getInt("output") > 0 if 1 else 0)
+        if json_val.has("pulseTimer"):
+            self._pulseTimer = json_val.getLong("pulseTimer")
+        if json_val.has("delayedPulseTimer"):
+            subjson = json_val.getYJSONObject("delayedPulseTimer");
             self._delayedPulseTimer = {"moving": None, "target": None, "ms": None}
-            for submemb in member.members:
-                if submemb.name == "moving":
-                    self._delayedPulseTimer["moving"] = submemb.ivalue
-                elif submemb.name == "target":
-                    self._delayedPulseTimer["target"] = submemb.ivalue
-                elif submemb.name == "ms":
-                    self._delayedPulseTimer["ms"] = submemb.ivalue
-            return 1
-        if member.name == "countdown":
-            self._countdown = member.ivalue
-            return 1
-        super(YRelay, self)._parseAttr(member)
+            if subjson.has("moving"):
+                self._delayedPulseTimer["moving"] = subjson.getInt("moving")
+            if subjson.has("target"):
+                self._delayedPulseTimer["target"] = subjson.getInt("target")
+            if subjson.has("ms"):
+                self._delayedPulseTimer["ms"] = subjson.getInt("ms")
+        if json_val.has("countdown"):
+            self._countdown = json_val.getLong("countdown")
+        super(YRelay, self)._parseAttr(json_val)
 
     def get_state(self):
         """
@@ -139,10 +132,12 @@ class YRelay(YFunction):
 
         On failure, throws an exception or returns YRelay.STATE_INVALID.
         """
+        # res
         if self._cacheExpiration <= YAPI.GetTickCount():
-            if self.load(YAPI.DefaultCacheValidity) != YAPI.SUCCESS:
+            if self.load(YAPI._yapiContext.GetCacheValidity()) != YAPI.SUCCESS:
                 return YRelay.STATE_INVALID
-        return self._state
+        res = self._state
+        return res
 
     def set_state(self, newval):
         """
@@ -160,28 +155,32 @@ class YRelay(YFunction):
 
     def get_stateAtPowerOn(self):
         """
-        Returns the state of the relays at device startup (A for the idle position, B for the active
-        position, UNCHANGED for no change).
+        Returns the state of the relays at device startup (A for the idle position,
+        B for the active position, UNCHANGED to leave the relay state as is).
 
         @return a value among YRelay.STATEATPOWERON_UNCHANGED, YRelay.STATEATPOWERON_A and
-        YRelay.STATEATPOWERON_B corresponding to the state of the relays at device startup (A for the idle
-        position, B for the active position, UNCHANGED for no change)
+        YRelay.STATEATPOWERON_B corresponding to the state of the relays at device startup (A for the idle position,
+                B for the active position, UNCHANGED to leave the relay state as is)
 
         On failure, throws an exception or returns YRelay.STATEATPOWERON_INVALID.
         """
+        # res
         if self._cacheExpiration <= YAPI.GetTickCount():
-            if self.load(YAPI.DefaultCacheValidity) != YAPI.SUCCESS:
+            if self.load(YAPI._yapiContext.GetCacheValidity()) != YAPI.SUCCESS:
                 return YRelay.STATEATPOWERON_INVALID
-        return self._stateAtPowerOn
+        res = self._stateAtPowerOn
+        return res
 
     def set_stateAtPowerOn(self, newval):
         """
-        Preset the state of the relays at device startup (A for the idle position,
-        B for the active position, UNCHANGED for no modification). Remember to call the matching module saveToFlash()
+        Changes the state of the relays at device startup (A for the idle position,
+        B for the active position, UNCHANGED to leave the relay state as is).
+        Remember to call the matching module saveToFlash()
         method, otherwise this call will have no effect.
 
         @param newval : a value among YRelay.STATEATPOWERON_UNCHANGED, YRelay.STATEATPOWERON_A and
-        YRelay.STATEATPOWERON_B
+        YRelay.STATEATPOWERON_B corresponding to the state of the relays at device startup (A for the idle position,
+                B for the active position, UNCHANGED to leave the relay state as is)
 
         @return YAPI.SUCCESS if the call succeeds.
 
@@ -192,24 +191,30 @@ class YRelay(YFunction):
 
     def get_maxTimeOnStateA(self):
         """
-        Retourne the maximum time (ms) allowed for $THEFUNCTIONS$ to stay in state A before automatically
-        switching back in to B state. Zero means no maximum time.
+        Returns the maximum time (ms) allowed for the relay to stay in state
+        A before automatically switching back in to B state. Zero means no time limit.
 
-        @return an integer
+        @return an integer corresponding to the maximum time (ms) allowed for the relay to stay in state
+                A before automatically switching back in to B state
 
         On failure, throws an exception or returns YRelay.MAXTIMEONSTATEA_INVALID.
         """
+        # res
         if self._cacheExpiration <= YAPI.GetTickCount():
-            if self.load(YAPI.DefaultCacheValidity) != YAPI.SUCCESS:
+            if self.load(YAPI._yapiContext.GetCacheValidity()) != YAPI.SUCCESS:
                 return YRelay.MAXTIMEONSTATEA_INVALID
-        return self._maxTimeOnStateA
+        res = self._maxTimeOnStateA
+        return res
 
     def set_maxTimeOnStateA(self, newval):
         """
-        Sets the maximum time (ms) allowed for $THEFUNCTIONS$ to stay in state A before automatically
-        switching back in to B state. Use zero for no maximum time.
+        Changes the maximum time (ms) allowed for the relay to stay in state A
+        before automatically switching back in to B state. Use zero for no time limit.
+        Remember to call the saveToFlash()
+        method of the module if the modification must be kept.
 
-        @param newval : an integer
+        @param newval : an integer corresponding to the maximum time (ms) allowed for the relay to stay in state A
+                before automatically switching back in to B state
 
         @return YAPI.SUCCESS if the call succeeds.
 
@@ -220,24 +225,30 @@ class YRelay(YFunction):
 
     def get_maxTimeOnStateB(self):
         """
-        Retourne the maximum time (ms) allowed for $THEFUNCTIONS$ to stay in state B before automatically
-        switching back in to A state. Zero means no maximum time.
+        Retourne the maximum time (ms) allowed for the relay to stay in state B
+        before automatically switching back in to A state. Zero means no time limit.
 
         @return an integer
 
         On failure, throws an exception or returns YRelay.MAXTIMEONSTATEB_INVALID.
         """
+        # res
         if self._cacheExpiration <= YAPI.GetTickCount():
-            if self.load(YAPI.DefaultCacheValidity) != YAPI.SUCCESS:
+            if self.load(YAPI._yapiContext.GetCacheValidity()) != YAPI.SUCCESS:
                 return YRelay.MAXTIMEONSTATEB_INVALID
-        return self._maxTimeOnStateB
+        res = self._maxTimeOnStateB
+        return res
 
     def set_maxTimeOnStateB(self, newval):
         """
-        Sets the maximum time (ms) allowed for $THEFUNCTIONS$ to stay in state B before automatically
-        switching back in to A state. Use zero for no maximum time.
+        Changes the maximum time (ms) allowed for the relay to stay in state B before
+        automatically switching back in to A state. Use zero for no time limit.
+        Remember to call the saveToFlash()
+        method of the module if the modification must be kept.
 
-        @param newval : an integer
+        @param newval : an integer corresponding to the maximum time (ms) allowed for the relay to stay in
+        state B before
+                automatically switching back in to A state
 
         @return YAPI.SUCCESS if the call succeeds.
 
@@ -255,10 +266,12 @@ class YRelay(YFunction):
 
         On failure, throws an exception or returns YRelay.OUTPUT_INVALID.
         """
+        # res
         if self._cacheExpiration <= YAPI.GetTickCount():
-            if self.load(YAPI.DefaultCacheValidity) != YAPI.SUCCESS:
+            if self.load(YAPI._yapiContext.GetCacheValidity()) != YAPI.SUCCESS:
                 return YRelay.OUTPUT_INVALID
-        return self._output
+        res = self._output
+        return res
 
     def set_output(self, newval):
         """
@@ -285,10 +298,12 @@ class YRelay(YFunction):
 
         On failure, throws an exception or returns YRelay.PULSETIMER_INVALID.
         """
+        # res
         if self._cacheExpiration <= YAPI.GetTickCount():
-            if self.load(YAPI.DefaultCacheValidity) != YAPI.SUCCESS:
+            if self.load(YAPI._yapiContext.GetCacheValidity()) != YAPI.SUCCESS:
                 return YRelay.PULSETIMER_INVALID
-        return self._pulseTimer
+        res = self._pulseTimer
+        return res
 
     def set_pulseTimer(self, newval):
         rest_val = str(newval)
@@ -299,7 +314,7 @@ class YRelay(YFunction):
         Sets the relay to output B (active) for a specified duration, then brings it
         automatically back to output A (idle state).
 
-        @param ms_duration : pulse duration, in millisecondes
+        @param ms_duration : pulse duration, in milliseconds
 
         @return YAPI.SUCCESS if the call succeeds.
 
@@ -309,10 +324,12 @@ class YRelay(YFunction):
         return self._setAttr("pulseTimer", rest_val)
 
     def get_delayedPulseTimer(self):
+        # res
         if self._cacheExpiration <= YAPI.GetTickCount():
-            if self.load(YAPI.DefaultCacheValidity) != YAPI.SUCCESS:
+            if self.load(YAPI._yapiContext.GetCacheValidity()) != YAPI.SUCCESS:
                 return YRelay.DELAYEDPULSETIMER_INVALID
-        return self._delayedPulseTimer
+        res = self._delayedPulseTimer
+        return res
 
     def set_delayedPulseTimer(self, newval):
         rest_val = str(newval.target) + ":" + str(newval.ms)
@@ -322,8 +339,8 @@ class YRelay(YFunction):
         """
         Schedules a pulse.
 
-        @param ms_delay : waiting time before the pulse, in millisecondes
-        @param ms_duration : pulse duration, in millisecondes
+        @param ms_delay : waiting time before the pulse, in milliseconds
+        @param ms_duration : pulse duration, in milliseconds
 
         @return YAPI.SUCCESS if the call succeeds.
 
@@ -342,10 +359,12 @@ class YRelay(YFunction):
 
         On failure, throws an exception or returns YRelay.COUNTDOWN_INVALID.
         """
+        # res
         if self._cacheExpiration <= YAPI.GetTickCount():
-            if self.load(YAPI.DefaultCacheValidity) != YAPI.SUCCESS:
+            if self.load(YAPI._yapiContext.GetCacheValidity()) != YAPI.SUCCESS:
                 return YRelay.COUNTDOWN_INVALID
-        return self._countdown
+        res = self._countdown
+        return res
 
     @staticmethod
     def FindRelay(func):
@@ -368,7 +387,12 @@ class YRelay(YFunction):
         found is returned. The search is performed first by hardware name,
         then by logical name.
 
-        @param func : a string that uniquely characterizes the relay
+        If a call to this object's is_online() method returns FALSE although
+        you are certain that the matching device is plugged, make sure that you did
+        call registerHub() at application initialization time.
+
+        @param func : a string that uniquely characterizes the relay, for instance
+                YLTCHRL1.relay1.
 
         @return a YRelay object allowing you to drive the relay.
         """
@@ -379,9 +403,41 @@ class YRelay(YFunction):
             YFunction._AddToCache("Relay", func, obj)
         return obj
 
+    def toggle(self):
+        """
+        Switch the relay to the opposite state.
+
+        @return YAPI.SUCCESS if the call succeeds.
+
+        On failure, throws an exception or returns a negative error code.
+        """
+        # sta
+        # fw
+        # mo
+        if self._firm == 0:
+            mo = self.get_module()
+            fw = mo.get_firmwareRelease()
+            if fw == YModule.FIRMWARERELEASE_INVALID:
+                return YRelay.STATE_INVALID
+            self._firm = YAPI._atoi(fw)
+        if self._firm < 34921:
+            sta = self.get_state()
+            if sta == YRelay.STATE_INVALID:
+                return YRelay.STATE_INVALID
+            if sta == YRelay.STATE_B:
+                self.set_state(YRelay.STATE_A)
+            else:
+                self.set_state(YRelay.STATE_B)
+            return YAPI.SUCCESS
+        else:
+            return self._setAttr("state","X")
+
     def nextRelay(self):
         """
         Continues the enumeration of relays started using yFirstRelay().
+        Caution: You can't make any assumption about the returned relays order.
+        If you want to find a specific a relay, use Relay.findRelay()
+        and a hardwareID or a logical name.
 
         @return a pointer to a YRelay object, corresponding to
                 a relay currently online, or a None pointer
@@ -396,7 +452,7 @@ class YRelay(YFunction):
 
 #--- (end of YRelay implementation)
 
-#--- (Relay functions)
+#--- (YRelay functions)
 
     @staticmethod
     def FirstRelay():
@@ -430,4 +486,4 @@ class YRelay(YFunction):
 
         return YRelay.FindRelay(serialRef.value + "." + funcIdRef.value)
 
-#--- (end of Relay functions)
+#--- (end of YRelay functions)
